@@ -1,6 +1,6 @@
 /*
- *  @(#) $Id: app.c 14626 2013-02-11 08:54:08Z yeti-dn $
- *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
+ *  @(#) $Id: app.c 15359 2013-09-22 10:20:58Z yeti-dn $
+ *  Copyright (C) 2003-2006,2013 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -21,6 +21,7 @@
 #include "config.h"
 #include <string.h>
 #include <stdarg.h>
+#include <math.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
@@ -40,6 +41,17 @@ enum {
     ITEM_REALSQUARE
 };
 
+typedef enum {
+    BRICK_PREVIEW_MEAN,
+    BRICK_PREVIEW_MINIMUM,
+    BRICK_PREVIEW_MAXIMUM,
+    BRICK_PREVIEW_MINPOS,
+    BRICK_PREVIEW_MAXPOS,
+    BRICK_PREVIEW_RMS,
+    BRICK_PREVIEW_CHANNEL,
+    BRICK_PREVIEW_SECTION,
+} BrickPreviewType;
+
 static GtkWidget *gwy_app_main_window = NULL;
 
 static GwyTool* current_tool = NULL;
@@ -48,8 +60,6 @@ static GQuark corner_item_quark = 0;
 static gboolean   gwy_app_main_window_save_position   (void);
 static void       gwy_app_main_window_restore_position(void);
 static gboolean   gwy_app_confirm_quit                (void);
-static void       gather_unsaved_cb                   (GwyDataWindow *data_window,
-                                                       GSList **unsaved);
 static gboolean   gwy_app_confirm_quit_dialog         (GSList *unsaved);
 static GtkWidget* gwy_app_menu_data_popup_create      (GtkAccelGroup *accel_group);
 static GtkWidget* gwy_app_menu_data_corner_create     (GtkAccelGroup *accel_group);
@@ -70,6 +80,13 @@ static void       gwy_app_graph_popup_menu_popup_key  (GtkWidget *menu,
                                                        GtkWidget *graph);
 static void       gwy_app_3d_window_export            (Gwy3DWindow *window);
 static void       gwy_app_3d_window_set_defaults      (Gwy3DWindow *window);
+static void       change_brick_preview                (GwyDataWindow *data_window);
+static GtkWidget* gwy_app_menu_brick_popup_create     (GtkAccelGroup *accel_group);
+static gboolean   gwy_app_brick_popup_menu_popup_mouse(GtkWidget *menu,
+                                                       GdkEventButton *event,
+                                                       GwyDataView *data_view);
+static void       gwy_app_brick_popup_menu_popup_key  (GtkWidget *menu,
+                                                       GtkWidget *data_window);
 static void       gwy_app_save_3d_export              (GtkWidget *dialog,
                                                        gint response,
                                                        Gwy3DWindow *gwy3dwindow);
@@ -82,7 +99,9 @@ static gboolean   gwy_app_3d_window_data2_filter      (GwyContainer *data2,
                                                        gint id2,
                                                        gpointer user_data);
 static void       gwy_app_data_window_reset_zoom      (void);
-static void       gwy_app_change_mask_color_cb        (void);
+static void       gwy_app_volume_window_reset_zoom    (void);
+static void       metadata_browser                    (gpointer pwhat);
+static void       gwy_app_change_mask_color           (void);
 
 /*****************************************************************************
  *                                                                           *
@@ -212,7 +231,6 @@ gwy_app_confirm_quit(void)
     GSList *unsaved = NULL;
     gboolean ok G_GNUC_UNUSED;
 
-    /* gwy_app_data_window_foreach((GFunc)gather_unsaved_cb, &unsaved); */
     if (!unsaved)
         return TRUE;
     ok = gwy_app_confirm_quit_dialog(unsaved);
@@ -221,17 +239,6 @@ gwy_app_confirm_quit(void)
     return TRUE;
 }
 
-/* TODO: We must gather containers (== files), not windows (== channels) */
-static void
-gather_unsaved_cb(GwyDataWindow *data_window,
-                  GSList **unsaved)
-{
-    GwyContainer *data = gwy_data_window_get_data(data_window);
-
-    if (gwy_undo_container_get_modified(data))
-        *unsaved = g_slist_prepend(*unsaved, data_window);
-}
-
 static gboolean
 gwy_app_confirm_quit_dialog(GSList *unsaved)
 {
@@ -281,40 +288,8 @@ gwy_app_confirm_quit_dialog(GSList *unsaved)
 void
 _gwy_app_data_view_set_current(GwyDataView *data_view)
 {
-    GwyMenuSensFlags mask = (GWY_MENU_FLAG_DATA
-                             | GWY_MENU_FLAG_UNDO
-                             | GWY_MENU_FLAG_REDO
-                             | GWY_MENU_FLAG_DATA_MASK
-                             | GWY_MENU_FLAG_DATA_SHOW);
-    GwyMenuSensFlags state = GWY_MENU_FLAG_DATA;
-    GwyPixmapLayer *layer;
-    GwyContainer *data;
-
-    gwy_debug("%p", data_view);
-
     if (current_tool)
         gwy_tool_data_switched(current_tool, data_view);
-
-    if (!data_view) {
-        gwy_app_sensitivity_set_state(mask, 0);
-        return;
-    }
-
-    g_return_if_fail(GWY_IS_DATA_VIEW(data_view));
-
-    data = gwy_data_view_get_data(data_view);
-    if (gwy_undo_container_has_undo(data))
-        state |= GWY_MENU_FLAG_UNDO;
-    if (gwy_undo_container_has_redo(data))
-        state |= GWY_MENU_FLAG_REDO;
-
-    layer = gwy_data_view_get_base_layer(data_view);
-    if (gwy_layer_basic_get_has_presentation(GWY_LAYER_BASIC(layer)))
-        state |= GWY_MENU_FLAG_DATA_SHOW;
-    if (gwy_data_view_get_alpha_layer(data_view))
-        state |= GWY_MENU_FLAG_DATA_MASK;
-
-    gwy_app_sensitivity_set_state(mask, state);
 }
 
 void
@@ -396,7 +371,7 @@ gwy_app_menu_data_popup_create(GtkAccelGroup *accel_group)
             "mask_remove", GDK_K, GDK_CONTROL_MASK
         },
         {
-            N_("Mask _Color..."),  gwy_app_change_mask_color_cb,
+            N_("Mask _Color..."),  gwy_app_change_mask_color,
             NULL, 0, 0
         },
         {
@@ -415,6 +390,11 @@ gwy_app_menu_data_popup_create(GtkAccelGroup *accel_group)
             N_("Zoom _1:1"), gwy_app_data_window_reset_zoom,
             NULL, 0, 0
         },
+        {
+            N_("Metadata _Browser..."),
+            metadata_browser, GUINT_TO_POINTER(GWY_APP_DATA_FIELD),
+            GDK_B, GDK_CONTROL_MASK | GDK_SHIFT_MASK
+        },
     };
     GwySensitivityGroup *sensgroup;
     GtkWidget *menu, *item;
@@ -448,7 +428,7 @@ gwy_app_menu_data_popup_create(GtkAccelGroup *accel_group)
         else {
             item = gtk_menu_item_new_with_mnemonic(_(menu_items[i].label));
 
-            if (menu_items[i].callback == gwy_app_change_mask_color_cb)
+            if (menu_items[i].callback == gwy_app_change_mask_color)
                 gwy_sensitivity_group_add_widget(sensgroup, item,
                                                  GWY_MENU_FLAG_DATA_MASK);
         }
@@ -708,27 +688,6 @@ _gwy_app_graph_window_setup(GwyGraphWindow *graph_window)
                              popup_menu);
 }
 
-/**
- * _gwy_app_graph_set_current:
- * @window: Graph, can be %NULL.
- *
- * Updates application state upon switch to new graph.
- **/
-void
-_gwy_app_graph_set_current(GwyGraph *graph)
-{
-    gwy_debug("%p", graph);
-
-    if (!graph) {
-        gwy_app_sensitivity_set_state(GWY_MENU_FLAG_GRAPH, 0);
-        return;
-    }
-
-    g_return_if_fail(GWY_IS_GRAPH(graph));
-
-    gwy_app_sensitivity_set_state(GWY_MENU_FLAG_GRAPH, GWY_MENU_FLAG_GRAPH);
-}
-
 static gboolean
 gwy_app_graph_popup_menu_popup_mouse(GtkWidget *menu,
                                      GdkEventButton *event,
@@ -1213,19 +1172,387 @@ _gwy_app_spectra_set_current(GwySpectra *spectra)
 {
     if (current_tool)
         gwy_tool_spectra_switched(current_tool, spectra);
+}
 
-    if (!spectra) {
-        /*
-        gwy_app_sensitivity_set_state(GWY_MENU_FLAG_SPECTRA, 0);
-         */
+/*****************************************************************************
+ *                                                                           *
+ *     Bricks                                                                *
+ *                                                                           *
+ *****************************************************************************/
+
+void
+_gwy_app_brick_window_setup(GwyDataWindow *data_window)
+{
+    static GtkWidget *popup_menu = NULL;
+
+    GtkAccelGroup *accel_group;
+    GwyDataView *data_view;
+    GtkWidget *main_window;
+    GtkWidget *vbox, *hbox, *button, *label;
+
+    if (!popup_menu
+        && (main_window = gwy_app_main_window_get())) {
+
+        g_return_if_fail(GTK_IS_WINDOW(main_window));
+        accel_group = GTK_ACCEL_GROUP(g_object_get_data(G_OBJECT(main_window),
+                                                        "accel_group"));
+
+        if (accel_group && !popup_menu) {
+            popup_menu = gwy_app_menu_brick_popup_create(accel_group);
+            gtk_widget_show_all(popup_menu);
+        }
+    }
+
+    gwy_app_add_main_accel_group(GTK_WINDOW(data_window));
+
+    data_view = gwy_data_window_get_data_view(data_window);
+    g_signal_connect_swapped(data_view, "button-press-event",
+                             G_CALLBACK(gwy_app_brick_popup_menu_popup_mouse),
+                             popup_menu);
+    g_signal_connect_swapped(data_window, "popup-menu",
+                             G_CALLBACK(gwy_app_brick_popup_menu_popup_key),
+                             popup_menu);
+
+    hbox = gtk_hbox_new(FALSE, 0);
+
+    vbox = gtk_bin_get_child(GTK_BIN(data_window));
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
+    gtk_box_reorder_child(GTK_BOX(vbox), hbox, 0);
+
+    button = gtk_button_new_with_mnemonic(_("_Change Preview"));
+    GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_FOCUS);
+    gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+    g_signal_connect_swapped(button, "clicked",
+                             G_CALLBACK(change_brick_preview), data_window);
+
+    label = gtk_label_new(NULL);
+    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+    gtk_label_set_ellipsize(GTK_LABEL(label), PANGO_ELLIPSIZE_END);
+    gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
+
+    g_object_set_data(G_OBJECT(data_window), "gwy-brick-info", label);
+}
+
+static gboolean
+brick_preview_filter(GwyContainer *data,
+                     gint id,
+                     gpointer user_data)
+{
+    GwyDataField *field;
+    GQuark key = gwy_app_get_data_key_for_id(id);
+    GwySIUnit *funit, *bunit;
+    GwyBrick *brick = (GwyBrick*)user_data;
+
+    field = GWY_DATA_FIELD(gwy_container_get_object(data, key));
+    g_return_val_if_fail(field, FALSE);
+
+    if (field->xres != brick->xres
+        || field->yres != brick->yres
+        || ABS(log(field->xreal/brick->xreal)) > 1e-6
+        || ABS(log(field->yreal/brick->yreal)) > 1e-6)
+        return FALSE;
+
+    bunit = gwy_brick_get_si_unit_x(brick);
+    funit = gwy_data_field_get_si_unit_xy(field);
+    if (!gwy_si_unit_equal(bunit, funit))
+        return FALSE;
+    bunit = gwy_brick_get_si_unit_y(brick);
+    if (!gwy_si_unit_equal(bunit, funit))
+        return FALSE;
+
+    return TRUE;
+}
+
+static void
+update_brick_preview_sens(BrickPreviewType type,
+                          GObject *dialog)
+{
+    GwyDataChooser *chooser;
+    GtkWidget *widget;
+    gboolean ok = TRUE;
+
+    widget = GTK_WIDGET(g_object_get_data(dialog, "channel-chooser"));
+    chooser = GWY_DATA_CHOOSER(widget);
+    gtk_widget_set_sensitive(widget, type == BRICK_PREVIEW_CHANNEL);
+
+    widget = GTK_WIDGET(g_object_get_data(dialog, "section-scale"));
+    gtk_widget_set_sensitive(widget, type == BRICK_PREVIEW_SECTION);
+    widget = GTK_WIDGET(g_object_get_data(dialog, "section-spin"));
+    gtk_widget_set_sensitive(widget, type == BRICK_PREVIEW_SECTION);
+    widget = GTK_WIDGET(g_object_get_data(dialog, "section-units"));
+    gtk_widget_set_sensitive(widget, type == BRICK_PREVIEW_SECTION);
+
+    if (type == BRICK_PREVIEW_CHANNEL) {
+        gint id;
+        GwyContainer *container = gwy_data_chooser_get_active(chooser, &id);
+        ok = !!container;
+    }
+    gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog), GTK_RESPONSE_OK, ok);
+}
+
+static void
+brick_preview_type_changed(GtkRadioButton *button,
+                           GObject *dialog)
+{
+    GSList *group = gtk_radio_button_get_group(button);
+    BrickPreviewType type = gwy_radio_buttons_get_current(group);
+    update_brick_preview_sens(type, dialog);
+}
+
+static void
+change_brick_preview(GwyDataWindow *data_window)
+{
+    GtkAdjustment *leveladj;
+    GtkWidget *dialog, *label, *table, *chooser, *scale, *spin;
+    GwyContainer *data, *cdata;
+    GwyDataField *preview;
+    GwyBrick *brick;
+    GSList *group, *l;
+    gint response, id, cid, i, level;
+    BrickPreviewType type;
+    gchar key[40];
+
+    gwy_app_data_browser_get_current(GWY_APP_BRICK, &brick,
+                                     GWY_APP_BRICK_ID, &id,
+                                     GWY_APP_CONTAINER, &data,
+                                     0);
+    g_return_if_fail(GWY_IS_BRICK(brick));
+    g_return_if_fail(id >= 0);
+
+    dialog = gtk_dialog_new_with_buttons(_("Change Volume Data Preview"),
+                                         GTK_WINDOW(data_window),
+                                         GTK_DIALOG_MODAL
+                                         | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                         GTK_STOCK_OK, GTK_RESPONSE_OK,
+                                         NULL);
+    table = gtk_table_new(5, 4, FALSE);
+    gtk_container_set_border_width(GTK_CONTAINER(table), 8);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), table, TRUE, TRUE, 0);
+    label = gtk_label_new(_("Preview quantity:"));
+    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
+    type = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(brick),
+                                              "gwy-preview-type"));
+    level = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(brick),
+                                              "gwy-preview-level"));
+    group = gwy_radio_buttons_createl(G_CALLBACK(brick_preview_type_changed),
+                                      dialog,
+                                      type,
+                                      _("Mean"), BRICK_PREVIEW_MEAN,
+                                      _("Minimum"), BRICK_PREVIEW_MINIMUM,
+                                      _("Maximum"), BRICK_PREVIEW_MAXIMUM,
+                                      _("Min. position"), BRICK_PREVIEW_MINPOS,
+                                      _("Max. position"), BRICK_PREVIEW_MAXPOS,
+                                      _("RMS"), BRICK_PREVIEW_RMS,
+                                      _("Channel:"), BRICK_PREVIEW_CHANNEL,
+                                      _("Section:"), BRICK_PREVIEW_SECTION,
+                                      NULL);
+    l = group;
+    i = 1;
+    while (l) {
+        gtk_table_attach(GTK_TABLE(table), GTK_WIDGET(l->data),
+                         0, 1, i, i+1, GTK_FILL, 0, 0, 0);
+        i++;
+        l = g_slist_next(l);
+    }
+    chooser = gwy_data_chooser_new_channels();
+    gwy_data_chooser_set_filter(GWY_DATA_CHOOSER(chooser),
+                                &brick_preview_filter, brick, NULL);
+    gtk_table_attach(GTK_TABLE(table), chooser,
+                     1, 4, 7, 8, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+    g_object_set_data(G_OBJECT(dialog), "channel-chooser", chooser);
+
+    leveladj = (GtkAdjustment*)gtk_adjustment_new(level, 0, brick->zres-1,
+                                                  1, 10, 0);
+    scale = gtk_hscale_new(leveladj);
+    gtk_scale_set_draw_value(GTK_SCALE(scale), FALSE);
+    gtk_widget_set_size_request(scale, 120, -1);
+    gtk_table_attach(GTK_TABLE(table), scale,
+                     1, 2, 8, 9, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+    g_object_set_data(G_OBJECT(dialog), "section-scale", scale);
+    spin = gtk_spin_button_new(leveladj, 0.0, 0);
+    gtk_table_attach(GTK_TABLE(table), spin, 2, 3, 8, 9, GTK_FILL, 0, 0, 0);
+    g_object_set_data(G_OBJECT(dialog), "section-spin", spin);
+    label = gtk_label_new("px");
+    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+    gtk_table_attach(GTK_TABLE(table), label, 3, 4, 8, 9, GTK_FILL, 0, 0, 0);
+    g_object_set_data(G_OBJECT(dialog), "section-units", label);
+
+    update_brick_preview_sens(gwy_radio_buttons_get_current(group),
+                              G_OBJECT(dialog));
+
+    gtk_widget_show_all(dialog);
+    response = gtk_dialog_run(GTK_DIALOG(dialog));
+    type = gwy_radio_buttons_get_current(group);
+    cdata = gwy_data_chooser_get_active(GWY_DATA_CHOOSER(chooser), &cid);
+    level = gwy_adjustment_get_int(leveladj);
+    level = CLAMP(level, 0, brick->zres-1);
+    gtk_widget_destroy(dialog);
+    if (response != GTK_RESPONSE_OK)
+        return;
+
+    g_snprintf(key, sizeof(key), "/brick/%d/preview", id);
+    if (!gwy_container_gis_object_by_name(data, key, (GObject**)&preview)) {
+        g_warning("No preview field found for brick %d.", id);
         return;
     }
 
-    g_return_if_fail(GWY_IS_SPECTRA(spectra));
+    if (type == BRICK_PREVIEW_MEAN)
+        gwy_brick_mean_plane(brick, preview,
+                             0, 0, 0,
+                             brick->xres, brick->yres, -1,
+                             TRUE);
+    else if (type == BRICK_PREVIEW_MINIMUM)
+        gwy_brick_min_plane(brick, preview,
+                            0, 0, 0,
+                            brick->xres, brick->yres, -1,
+                            TRUE);
+    else if (type == BRICK_PREVIEW_MAXIMUM)
+        gwy_brick_max_plane(brick, preview,
+                            0, 0, 0,
+                            brick->xres, brick->yres, -1,
+                            TRUE);
+    else if (type == BRICK_PREVIEW_MINPOS)
+        gwy_brick_minpos_plane(brick, preview,
+                            0, 0, 0,
+                            brick->xres, brick->yres, -1,
+                            TRUE);
+    else if (type == BRICK_PREVIEW_MAXPOS)
+        gwy_brick_maxpos_plane(brick, preview,
+                            0, 0, 0,
+                            brick->xres, brick->yres, -1,
+                            TRUE);
+    else if (type == BRICK_PREVIEW_RMS)
+        gwy_brick_rms_plane(brick, preview,
+                            0, 0, 0,
+                            brick->xres, brick->yres, -1,
+                            TRUE);
+    else if (type == BRICK_PREVIEW_CHANNEL) {
+        GQuark quark = gwy_app_get_data_key_for_id(cid);
+        GObject *field = gwy_container_get_object(cdata, quark);
+        gwy_serializable_clone(field, G_OBJECT(preview));
+    }
+    else if (type == BRICK_PREVIEW_SECTION)
+        gwy_brick_extract_plane(brick, preview,
+                                0, 0, level,
+                                brick->xres, brick->yres, -1,
+                                TRUE);
+    else {
+        g_return_if_reached();
+    }
 
-    /*
-    gwy_app_sensitivity_set_state(GWY_MENU_FLAG_SPECTRA, GWY_MENU_FLAG_SPECTRA);
-    */
+    g_object_set_data(G_OBJECT(brick),
+                      "gwy-preview-type", GUINT_TO_POINTER(type));
+    g_object_set_data(G_OBJECT(brick),
+                      "gwy-preview-level", GUINT_TO_POINTER(level));
+    gwy_data_field_data_changed(preview);
+}
+
+static GtkWidget*
+gwy_app_menu_brick_popup_create(GtkAccelGroup *accel_group)
+{
+    static struct {
+        const gchar *label;
+        gpointer callback;
+        gpointer cbdata;
+        guint key;
+        GdkModifierType mods;
+    }
+    const menu_items[] = {
+        {
+            N_("Zoom _1:1"), gwy_app_volume_window_reset_zoom,
+            NULL, 0, 0
+        },
+        {
+            N_("Metadata _Browser..."),
+            metadata_browser, GUINT_TO_POINTER(GWY_APP_BRICK),
+            GDK_B, GDK_CONTROL_MASK | GDK_SHIFT_MASK
+        },
+    };
+    GwySensitivityGroup *sensgroup;
+    GtkWidget *menu, *item;
+    const gchar *name;
+    guint i, mask;
+
+    menu = gtk_menu_new();
+    if (accel_group)
+        gtk_menu_set_accel_group(GTK_MENU(menu), accel_group);
+    sensgroup = gwy_app_sensitivity_get_group();
+    for (i = 0; i < G_N_ELEMENTS(menu_items); i++) {
+        if (menu_items[i].callback == gwy_app_run_volume_func
+            && !gwy_volume_func_get_run_types((gchar*)menu_items[i].cbdata)) {
+            g_warning("Brick function <%s> for "
+                      "data view context menu is not available.",
+                      (const gchar*)menu_items[i].cbdata);
+            continue;
+        }
+        if (menu_items[i].callback == gwy_app_run_volume_func) {
+            name = _(gwy_volume_func_get_menu_path(menu_items[i].cbdata));
+            name = strrchr(name, '/');
+            if (!name) {
+                g_warning("Invalid translated menu path for <%s>",
+                          (const gchar*)menu_items[i].cbdata);
+                continue;
+            }
+            item = gtk_menu_item_new_with_mnemonic(name + 1);
+            mask = gwy_volume_func_get_sensitivity_mask(menu_items[i].cbdata);
+            gwy_sensitivity_group_add_widget(sensgroup, item, mask);
+        }
+        else {
+            item = gtk_menu_item_new_with_mnemonic(_(menu_items[i].label));
+        }
+
+        if (menu_items[i].key)
+            gtk_widget_add_accelerator(item, "activate", accel_group,
+                                       menu_items[i].key, menu_items[i].mods,
+                                       GTK_ACCEL_VISIBLE | GTK_ACCEL_LOCKED);
+        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+        g_signal_connect_swapped(item, "activate",
+                                 G_CALLBACK(menu_items[i].callback),
+                                 menu_items[i].cbdata);
+    }
+
+    return menu;
+}
+
+static gboolean
+gwy_app_brick_popup_menu_popup_mouse(GtkWidget *menu,
+                                     GdkEventButton *event,
+                                     GwyDataView *data_view)
+{
+    if (event->button != 3)
+        return FALSE;
+
+    gwy_app_data_browser_select_volume(data_view);
+    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
+                   event->button, event->time);
+
+    return TRUE;
+}
+
+static void
+gwy_app_brick_popup_menu_position(G_GNUC_UNUSED GtkMenu *menu,
+                                  gint *x,
+                                  gint *y,
+                                  gboolean *push_in,
+                                  GtkWidget *window)
+{
+    GwyDataView *data_view;
+
+    data_view = gwy_data_window_get_data_view(GWY_DATA_WINDOW(window));
+    gdk_window_get_origin(GTK_WIDGET(data_view)->window, x, y);
+    *push_in = TRUE;
+}
+
+static void
+gwy_app_brick_popup_menu_popup_key(GtkWidget *menu,
+                                   GtkWidget *data_window)
+{
+    gtk_menu_popup(GTK_MENU(menu), NULL, NULL,
+                   (GtkMenuPositionFunc)gwy_app_brick_popup_menu_position,
+                   data_window,
+                   0, gtk_get_current_event_time());
 }
 
 /*****************************************************************************
@@ -1243,9 +1570,11 @@ _gwy_app_spectra_set_current(GwySpectra *spectra)
 void
 gwy_app_switch_tool(const gchar *toolname)
 {
+    GwyContainer *settings;
     GwyTool *newtool;
     GwyDataView *data_view;
     GType type;
+    gboolean do_restore = FALSE;
 
     gwy_debug("%s", toolname ? toolname : "NONE");
     type = g_type_from_name(toolname);
@@ -1265,6 +1594,12 @@ gwy_app_switch_tool(const gchar *toolname)
     current_tool = newtool;
     g_return_if_fail(GWY_IS_TOOL(newtool));
 
+    settings = gwy_app_settings_get();
+    gwy_container_gis_boolean_by_name(settings, "/app/restore-tool-position",
+                                      &do_restore);
+    if (do_restore)
+        gwy_tool_restore_screen_position(newtool);
+
     if (data_view) {
         GwySpectra *spectra;
 
@@ -1287,7 +1622,49 @@ gwy_app_data_window_reset_zoom(void)
 }
 
 static void
-gwy_app_change_mask_color_cb(void)
+gwy_app_volume_window_reset_zoom(void)
+{
+    GtkWidget *window, *view;
+
+    gwy_app_data_browser_get_current(GWY_APP_VOLUME_VIEW, &view, 0);
+    window = gtk_widget_get_ancestor(view, GWY_TYPE_DATA_WINDOW);
+    g_return_if_fail(window);
+    gwy_data_window_set_zoom(GWY_DATA_WINDOW(window), 10000);
+}
+
+static void
+metadata_browser(gpointer pwhat)
+{
+    GwyAppWhat what = GPOINTER_TO_UINT(pwhat);
+    GtkWidget *view;
+    GwyContainer *container;
+    gint id;
+
+    if (what == GWY_APP_DATA_FIELD)
+        gwy_app_data_browser_get_current(GWY_APP_DATA_VIEW, &view,
+                                         GWY_APP_CONTAINER, &container,
+                                         GWY_APP_DATA_FIELD_ID, &id,
+                                         0);
+    else if (what == GWY_APP_BRICK)
+        gwy_app_data_browser_get_current(GWY_APP_VOLUME_VIEW, &view,
+                                         GWY_APP_CONTAINER, &container,
+                                         GWY_APP_BRICK_ID, &id,
+                                         0);
+    else {
+        g_return_if_reached();
+    }
+
+    if (!view || !container || id == -1)
+        return;
+
+    if (what == GWY_APP_DATA_FIELD)
+        gwy_app_metadata_browser_for_channel(container, id);
+    else if (what == GWY_APP_BRICK)
+        gwy_app_metadata_browser_for_volume(container, id);
+}
+
+static void
+gwy_app_change_mask_color(void)
 {
     GwyDataView *data_view;
 
