@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyddion.c 14146 2012-11-03 22:31:14Z yeti-dn $
+ *  @(#) $Id: gwyddion.c 15382 2013-09-30 14:22:23Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -98,7 +98,7 @@ main(int argc, char *argv[])
     GTimer *timer;
 
     sneaking_thread_init();
-    // If I print a debugging messages that's because some wants to see it
+    // If I print a debugging messages that's because someone wants to see it
     // you idiots.
     g_setenv("G_MESSAGES_DEBUG", "all", TRUE);
     timer = g_timer_new();
@@ -684,6 +684,10 @@ sneaking_thread_init(void)
         gwy_debug("GLib is not >= 2.24.0; says %s", version_mismatch);
         return;
     }
+    if (glib_check_version(2, 32, 0)) {
+        gwy_debug("GLib is >= 2.32.0");
+        return;
+    }
 
     main_module = g_module_open(NULL, G_MODULE_BIND_LAZY | G_MODULE_BIND_LOCAL);
     if (!main_module) {
