@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyplaintool.c 12576 2011-07-11 14:51:57Z yeti-dn $
+ *  @(#) $Id: gwyplaintool.c 15203 2013-08-23 18:56:57Z yeti-dn $
  *  Copyright (C) 2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -343,9 +343,12 @@ gwy_plain_tool_data_item_changed(GwyContainer *container,
             = g_signal_connect_swapped(plain_tool->data_field, "data-changed",
                                        G_CALLBACK(gwy_plain_tool_data_changed),
                                        plain_tool);
+        gwy_plain_tool_data_changed(plain_tool);
+    }
+    else {
+        /* No data field means we disconnect completely. */
+        gwy_tool_data_switched(GWY_TOOL(plain_tool), NULL);
     }
-
-    gwy_plain_tool_data_changed(plain_tool);
 }
 
 static void
