@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: menu.c 12592 2011-07-13 19:23:05Z yeti-dn $
+ *  @(#) $Id: menu.c 14848 2013-04-10 19:31:43Z yeti-dn $
  *  Copyright (C) 2003-2006,2011 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -62,6 +62,7 @@ static GQuark last_name_quark   = 0;
 static GtkWidget           *recent_files_menu = NULL;
 static GtkWidget           *process_menu      = NULL;
 static GtkWidget           *graph_menu        = NULL;
+static GtkWidget           *volume_menu       = NULL;
 static GtkTooltips         *app_tooltips      = NULL;
 static GwySensitivityGroup *app_sensgroup     = NULL;
 
@@ -541,7 +542,7 @@ gwy_app_menu_add_proc_func(const gchar *name,
  * Constructs the application <guimenu>Data Process</guimenu> menu.
  *
  * The menu is created from data processing functions registered by modules,
- * therefore module registeration has to be performed first for this function
+ * therefore module registration has to be performed first for this function
  * to make sense.
  *
  * Returns: A newly ceated data processing menu (a #GtkMenu).
@@ -583,7 +584,7 @@ gwy_app_menu_add_graph_func(const gchar *name,
  * Constructs the application <guimenu>Graph</guimenu> menu.
  *
  * The menu is created from graph functions registered by modules,
- * therefore module registeration has to be performed first for this function
+ * therefore module registration has to be performed first for this function
  * to make sense.
  *
  * Returns: A newly ceated graph menu (a #GtkMenu).
@@ -610,6 +611,50 @@ gwy_app_build_graph_menu(GtkAccelGroup *accel_group)
     return menu;
 }
 
+static void
+gwy_app_menu_add_volume_func(const gchar *name,
+                             GNode *root)
+{
+    gwy_app_menu_add_node(root, name,
+                          gwy_volume_func_get_menu_path(name),
+                          gwy_volume_func_get_stock_id(name));
+}
+
+/**
+ * gwy_app_build_volume_menu:
+ * @accel_group: Acceleration group to be associated with the menu.
+ *
+ * Constructs the application <guimenu>Volume Data</guimenu> menu.
+ *
+ * The menu is created from volume data processing functions registered by
+ * modules, therefore module registration has to be performed first for this
+ * function to make sense.
+ *
+ * Returns: A newly ceated data processing menu (a #GtkMenu).
+ *
+ * Since: 2.32
+ **/
+GtkWidget*
+gwy_app_build_volume_menu(GtkAccelGroup *accel_group)
+{
+    MenuNodeData *data;
+    GtkWidget *menu;
+    GNode *root;
+
+    data = g_new0(MenuNodeData, 1);
+    data->path = g_strdup("");
+    data->item_translated = g_strdup(_("_Volume Data"));
+    root = g_node_new(data);
+    gwy_volume_func_foreach((GFunc)&gwy_app_menu_add_volume_func, root);
+    menu = gwy_app_build_module_func_menu(root, "<volume>/Volume Data",
+                                          accel_group,
+                                          G_CALLBACK(gwy_app_run_volume_func),
+                                          gwy_volume_func_get_sensitivity_mask);
+    volume_menu = menu;
+
+    return menu;
+}
+
 /**
  * gwy_app_process_menu_add_run_last:
  * @menu: Menu created with gwy_app_build_process_menu().
@@ -805,6 +850,69 @@ gwy_app_run_graph_func(const gchar *name)
     gwy_graph_func_run(name, graph);
 }
 
+/**
+ * gwy_app_run_volume_func:
+ * @name: A volume data processing function name.
+ *
+ * Runs a volume data processing function on the current data.
+ *
+ * From the run modes function @name supports, the most interactive one is
+ * selected.
+ *
+ * Returns: The actually used mode (nonzero), or 0 on failure.
+ *
+ * Since: 2.32
+ **/
+GwyRunType
+gwy_app_run_volume_func(const gchar *name)
+{
+    GwyRunType run_types[] = { GWY_RUN_INTERACTIVE, GWY_RUN_IMMEDIATE, };
+    GwyRunType available_run_modes;
+    gsize i;
+
+    gwy_debug("`%s'", name);
+    available_run_modes = gwy_volume_func_get_run_types(name);
+    g_return_val_if_fail(available_run_modes, 0);
+    for (i = 0; i < G_N_ELEMENTS(run_types); i++) {
+        if (run_types[i] & available_run_modes) {
+            gwy_app_run_volume_func_in_mode(name, run_types[i]);
+            return run_types[i];
+        }
+    }
+    return 0;
+}
+
+/**
+ * gwy_app_run_volume_func_in_mode:
+ * @name: A volume data processing function name.
+ * @run: A run mode.
+ *
+ * Runs a volume data processing function on current data in specified mode.
+ *
+ * Since: 2.32
+ **/
+void
+gwy_app_run_volume_func_in_mode(const gchar *name,
+                                GwyRunType run)
+{
+    GwyContainer *data;
+
+    gwy_debug("`%s'", name);
+    if (!(run & gwy_volume_func_get_run_types(name)))
+        return;
+
+    gwy_app_data_browser_get_current(GWY_APP_CONTAINER, &data, 0);
+    g_return_if_fail(data
+                     || !(gwy_volume_func_get_sensitivity_mask(name)
+                          & GWY_MENU_FLAG_DATA));
+    gwy_volume_func_run(name, data, run);
+    /*
+    gwy_app_update_last_volume_func(name);
+    gwy_app_sensitivity_set_state(GWY_MENU_FLAG_LAST_PROC,
+                                  GWY_MENU_FLAG_LAST_PROC);
+                                  */
+}
+
 static void
 gwy_app_recent_file_list_cb(void)
 {
@@ -1162,17 +1270,19 @@ gwy_app_get_tooltips(void)
 
 /**
  * GwyMenuSensFlags:
- * @GWY_MENU_FLAG_DATA: There's at least a one data window present.
- * @GWY_MENU_FLAG_UNDO: There's something to undo (for current data window).
- * @GWY_MENU_FLAG_REDO: There's something to redo (for current data window).
- * @GWY_MENU_FLAG_GRAPH: There's at least a one graph window present.
+ * @GWY_MENU_FLAG_DATA: There is at least a one data window present.
+ * @GWY_MENU_FLAG_UNDO: There is something to undo (for current data window).
+ * @GWY_MENU_FLAG_REDO: There is something to redo (for current data window).
+ * @GWY_MENU_FLAG_GRAPH: There is at least a one graph window present.
  * @GWY_MENU_FLAG_LAST_PROC: There is a last-run data processing function
  *                           to rerun.
  * @GWY_MENU_FLAG_LAST_GRAPH: There is a last-run graph function to rerun.
  * @GWY_MENU_FLAG_DATA_MASK: There is a mask on the data.
  * @GWY_MENU_FLAG_DATA_SHOW: There is a presentation on the data.
  * @GWY_MENU_FLAG_3D: A 3D view is present.
- * @GWY_MENU_FLAG_FILE: A file is open, with any type of data (Since: 2.5).
+ * @GWY_MENU_FLAG_FILE: A file is open, with any type of data (Since 2.5).
+ * @GWY_MENU_FLAG_VOLUME: There is at least one volume data window present
+ *                        (Since 2.32).
  * @GWY_MENU_FLAG_MASK: All the bits combined.
  *
  * Global application sensitivity flags.
