@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: settings.c 12596 2011-07-13 20:31:23Z yeti-dn $
+ *  @(#) $Id: settings.c 14850 2013-04-10 19:50:40Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -483,7 +483,7 @@ gchar**
 gwy_app_settings_get_module_dirs(void)
 {
     const gchar *module_types[] = {
-        "layer", "file", "process", "graph", "tool"
+        "layer", "file", "process", "graph", "tool", "volume"
     };
     gchar **module_dirs;
     gchar *p;
