@@ -1,6 +1,6 @@
 /*
- *  @(#) $Id: toolbox.c 13775 2012-09-06 08:43:18Z yeti-dn $
- *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
+ *  @(#) $Id: toolbox.c 15026 2013-05-13 13:11:11Z yeti-dn $
+ *  Copyright (C) 2003-2006,2013 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -57,7 +57,8 @@ typedef enum {
     GWY_APP_FUNC_TYPE_BUILTIN,
     GWY_APP_FUNC_TYPE_PROC,
     GWY_APP_FUNC_TYPE_GRAPH,
-    GWY_APP_FUNC_TYPE_TOOL
+    GWY_APP_FUNC_TYPE_TOOL,
+    GWY_APP_FUNC_TYPE_VOLUME
 } GwyAppFuncType;
 
 typedef struct {
@@ -83,7 +84,6 @@ static void       toolbox_dnd_data_received    (GtkWidget *widget,
                                                 guint info,
                                                 guint time_,
                                                 gpointer user_data);
-static void       gwy_app_meta_browser         (void);
 static void       delete_app_window            (void);
 static void       gwy_app_zoom_set_cb          (gpointer user_data);
 static void       gwy_app_undo_cb              (void);
@@ -105,7 +105,7 @@ static GtkTargetEntry dnd_target_table[] = {
 
 /* Translatability hack, intltool seems overkill at this point. */
 #define GWY_TOOLBOX_IGNORE(x) /* */
-GWY_TOOLBOX_IGNORE((_("View"), _("Data Process"), _("Graph"), _("Tools")))
+GWY_TOOLBOX_IGNORE((_("View"), _("Data Process"), _("Graph"), _("Tools"), _("Volume")))
 
 /* FIXME: A temporary hack. */
 static void
@@ -304,6 +304,7 @@ toolbox_ui_start_item(GwyAppToolboxBuilder *builder,
         { "builtin", GWY_APP_FUNC_TYPE_BUILTIN, },
         { "proc",    GWY_APP_FUNC_TYPE_PROC,    },
         { "graph",   GWY_APP_FUNC_TYPE_GRAPH,   },
+        { "volume",  GWY_APP_FUNC_TYPE_VOLUME,  },
         { "tool",    GWY_APP_FUNC_TYPE_TOOL,    },
     };
 
@@ -379,6 +380,17 @@ toolbox_ui_start_item(GwyAppToolboxBuilder *builder,
         action.sens = gwy_graph_func_get_sensitivity_mask(func);
         break;
 
+        case GWY_APP_FUNC_TYPE_VOLUME:
+        if (!gwy_graph_func_exists(func)) {
+            g_warning("Function volume::%s does not exist", func);
+            return;
+        }
+        action.stock_id = gwy_volume_func_get_stock_id(func);
+        action.tooltip = gwy_volume_func_get_tooltip(func);
+        action.callback = G_CALLBACK(gwy_app_run_volume_func);
+        action.sens = gwy_volume_func_get_sensitivity_mask(func);
+        break;
+
         case GWY_APP_FUNC_TYPE_TOOL:
         /* Handle unseen tools */
         if (!func) {
@@ -617,6 +629,9 @@ gwy_app_toolbox_create(void)
     menu = gwy_app_build_graph_menu(accel_group);
     toolbox_add_menubar(container, menu, _("_Graph"));
 
+    menu = gwy_app_build_volume_menu(accel_group);
+    toolbox_add_menubar(container, menu, _("_Volume Data"));
+
     toolbox_add_menubar(container,
                         gwy_app_menu_create_meta_menu(accel_group), _("_Meta"));
 
@@ -668,14 +683,6 @@ gwy_app_menu_create_meta_menu(GtkAccelGroup *accel_group)
             "<Item>",
             NULL
         },
-        {
-            N_("/_Metadata Browser"),
-            NULL,
-            gwy_app_meta_browser,
-            0,
-            "<Item>",
-            NULL
-        },
         {
             N_("/Module _Browser"),
             NULL,
@@ -719,10 +726,6 @@ gwy_app_menu_create_meta_menu(GtkAccelGroup *accel_group)
     gtk_item_factory_create_items(item_factory,
                                   G_N_ELEMENTS(menu_items), menu_items, NULL);
 
-    set_sensitivity(item_factory,
-                    "<meta>/Metadata Browser", GWY_MENU_FLAG_DATA,
-                    NULL);
-
     return gtk_item_factory_get_widget(item_factory, "<meta>");
 }
 
@@ -1034,18 +1037,6 @@ toolbox_dnd_data_received(G_GNUC_UNUSED GtkWidget *widget,
         g_ptr_array_free(files, TRUE);
 }
 
-static void
-gwy_app_meta_browser(void)
-{
-    GwyContainer *data;
-    gint id;
-
-    gwy_app_data_browser_get_current(GWY_APP_CONTAINER, &data,
-                                     GWY_APP_DATA_FIELD_ID, &id,
-                                     0);
-    gwy_app_metadata_browser(data, id);
-}
-
 static void
 delete_app_window(void)
 {
