@@ -1,6 +1,6 @@
 /*
- *  @(#) $Id: validate.c 12576 2011-07-11 14:51:57Z yeti-dn $
- *  Copyright (C) 2007 David Necas (Yeti).
+ *  @(#) $Id: validate.c 15022 2013-05-12 20:12:49Z yeti-dn $
+ *  Copyright (C) 2007-2013 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -43,6 +43,7 @@ typedef struct {
     GArray *channels;
     GArray *graphs;
     GArray *spectra;
+    GArray *volumes;
 } GwyDataValidationInfo;
 
 static GwyDataValidationFailure*
@@ -228,6 +229,7 @@ validate_item_pass1(gpointer hash_key,
         case KEY_IS_MASK:
         case KEY_IS_SHOW:
         case KEY_IS_CALDATA:
+        case KEY_IS_BRICK_PREVIEW:
         check_type(gvalue, GWY_TYPE_DATA_FIELD, key, errors);
         break;
 
@@ -241,7 +243,13 @@ validate_item_pass1(gpointer hash_key,
             g_array_append_val(info->spectra, id);
         break;
 
-        case KEY_IS_META:
+        case KEY_IS_BRICK:
+        if (check_type(gvalue, GWY_TYPE_BRICK, key, errors))
+            g_array_append_val(info->volumes, id);
+        break;
+
+        case KEY_IS_CHANNEL_META:
+        case KEY_IS_BRICK_META:
         check_type(gvalue, GWY_TYPE_CONTAINER, key, errors);
         break;
 
@@ -249,6 +257,8 @@ validate_item_pass1(gpointer hash_key,
         case KEY_IS_PALETTE:
         case KEY_IS_3D_PALETTE:
         case KEY_IS_3D_MATERIAL:
+        case KEY_IS_BRICK_TITLE:
+        case KEY_IS_BRICK_PREVIEW_PALETTE:
         check_type(gvalue, G_TYPE_STRING, key, errors);
         break;
 
@@ -270,6 +280,7 @@ validate_item_pass1(gpointer hash_key,
         case KEY_IS_DATA_VISIBLE:
         case KEY_IS_GRAPH_VISIBLE:
         case KEY_IS_SPECTRA_VISIBLE:
+        case KEY_IS_BRICK_VISIBLE:
         check_type(gvalue, G_TYPE_BOOLEAN, key, errors);
         break;
 
@@ -304,11 +315,10 @@ in_array(GArray *array,
 
 static void
 validate_item_pass2(gpointer hash_key,
-                    gpointer hash_value,
+                    G_GNUC_UNUSED gpointer hash_value,
                     gpointer user_data)
 {
     GQuark key = GPOINTER_TO_UINT(hash_key);
-    GValue *gvalue = (GValue*)hash_value;
     GwyDataValidationInfo *info = (GwyDataValidationInfo*)user_data;
     GSList **errors;
     const gchar *strkey;
@@ -326,7 +336,7 @@ validate_item_pass2(gpointer hash_key,
     switch (type) {
         case KEY_IS_MASK:
         case KEY_IS_SHOW:
-        case KEY_IS_META:
+        case KEY_IS_CHANNEL_META:
         case KEY_IS_TITLE:
         case KEY_IS_PALETTE:
         case KEY_IS_3D_PALETTE:
@@ -358,7 +368,6 @@ validate_item_pass2(gpointer hash_key,
         break;
 
         case KEY_IS_SPECTRA_VISIBLE:
-        check_type(gvalue, G_TYPE_BOOLEAN, key, errors);
         if (!in_array(info->spectra, id))
             *errors = g_slist_prepend(*errors,
                                       FAIL(GWY_DATA_ERROR_STRAY_SECONDARY_DATA,
@@ -367,6 +376,18 @@ validate_item_pass2(gpointer hash_key,
                                            id, strkey));
         break;
 
+        case KEY_IS_BRICK_VISIBLE:
+        case KEY_IS_BRICK_TITLE:
+        case KEY_IS_BRICK_PREVIEW:
+        case KEY_IS_BRICK_PREVIEW_PALETTE:
+        case KEY_IS_BRICK_META:
+        if (!in_array(info->volumes, id))
+            *errors = g_slist_prepend(*errors,
+                                      FAIL(GWY_DATA_ERROR_STRAY_SECONDARY_DATA,
+                                           key,
+                                           _("no brick %d exists for %s"),
+                                           id, strkey));
+
         default:
         break;
     }
@@ -557,6 +578,7 @@ gwy_data_validate(GwyContainer *data,
     info.channels = g_array_new(FALSE, FALSE, sizeof(gint));
     info.graphs = g_array_new(FALSE, FALSE, sizeof(gint));
     info.spectra = g_array_new(FALSE, FALSE, sizeof(gint));
+    info.volumes = g_array_new(FALSE, FALSE, sizeof(gint));
 
     gwy_container_foreach(data, NULL, &validate_item_pass1, &info);
     gwy_container_foreach(data, NULL, &validate_item_pass2, &info);
@@ -572,6 +594,7 @@ gwy_data_validate(GwyContainer *data,
     g_array_free(info.channels, TRUE);
     g_array_free(info.graphs, TRUE);
     g_array_free(info.spectra, TRUE);
+    g_array_free(info.volumes, TRUE);
 
     if (flags & GWY_DATA_VALIDATE_NO_REPORT) {
         gwy_data_validation_failure_list_free(errors);
