@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyddion.c 12576 2011-07-11 14:51:57Z yeti-dn $
+ *  @(#) $Id: gwyddion.c 15111 2013-07-31 06:17:07Z yeti-dn $
  *  Copyright (C) 2003-2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -41,7 +41,9 @@ gwy_type_init(void)
     if (types_initialized)
         return;
 
+#if (!GLIB_CHECK_VERSION(2, 36, 0))
     g_type_init();
+#endif
 
     g_type_class_peek(GWY_TYPE_SI_UNIT);
     g_type_class_peek(GWY_TYPE_CONTAINER);
