@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwynlfitpreset.c 12576 2011-07-11 14:51:57Z yeti-dn $
+ *  @(#) $Id: gwynlfitpreset.c 15041 2013-05-14 20:05:09Z klapetek $
  *  Copyright (C) 2000-2003 Martin Siler.
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
@@ -1712,7 +1712,7 @@ gwy_nlfit_preset_fit(GwyNLFitPreset *preset,
                                  preset->builtin->nparams, param,
                                  fixed_param, NULL, preset) >= 0.0;
 
-    if (ok && err) {
+    if (ok && err && fitter->covar) {
     /* FIXME: builtin */
         for (i = 0; i < preset->builtin->nparams; i++)
             err[i] = gwy_math_nlfit_get_sigma(fitter, i);
