@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyaxis.c 12576 2011-07-11 14:51:57Z yeti-dn $
+ *  @(#) $Id: gwyaxis.c 15068 2013-06-24 10:59:18Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -535,6 +535,10 @@ gwy_axis_realize(GtkWidget *widget)
                                     &attributes, attributes_mask);
     gdk_window_set_user_data(widget->window, widget);
 
+    /* Force widget to listen to relevant events.  Works around oxygen
+     * misdetection of ‘empty’ areas.  See KDE bug #317292. */
+    gtk_widget_add_events(widget, attributes.event_mask);
+
     widget->style = gtk_style_attach(widget->style, widget->window);
 
     /* set background to white forever */
@@ -1243,7 +1247,7 @@ gwy_axis_normalscale(GwyAxis *a)
             reqmax = 1.0;
         }
         else {
-            range = reqmax;
+            range = fabs(reqmax);
             reqmax += range/2.0;
             reqmin -= range/2.0;
         }
@@ -2034,11 +2038,16 @@ gwy_axis_export_vector(GwyAxis *axis, gint xmin, gint ymin,
     g_string_append_printf(out, "%%AxisLabel\n");
 
     plotlabel = g_string_new(axis->label_text->str);
-    units = gwy_si_unit_get_string(axis->unit, GWY_SI_UNIT_FORMAT_MARKUP);
+    units = gwy_si_unit_get_string(axis->unit, GWY_SI_UNIT_FORMAT_PLAIN);
     if (axis->magnification_string->len > 0 || *units) {
+
         g_string_append(plotlabel, " [");
-        if (axis->magnification_string->len)
-            g_string_append(plotlabel, axis->magnification_string->str);
+        if (axis->magnification_string->len) {
+            gchar *mag = gwy_strreplace(axis->magnification_string->str,
+                                        "µ", ")show /mu glyphshow (", (gsize)-1);
+            g_string_append(plotlabel, mag);
+            g_free(mag);
+        }
         else
             g_string_append(plotlabel, units);
         g_string_append(plotlabel, "]");
