@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwydatawindow.c 12576 2011-07-11 14:51:57Z yeti-dn $
+ *  @(#) $Id: gwydatawindow.c 15147 2013-08-10 11:47:31Z yeti-dn $
  *  Copyright (C) 2003,2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -797,6 +797,18 @@ gwy_data_window_resize_view(GwyDataWindow *data_window)
 {
     GtkRequisition req;
     GtkWidget *widget;
+    GwyPixmapLayer *layer;
+    const gchar *key;
+    GwyDataView *view;
+
+    if ((view = GWY_DATA_VIEW(data_window->data_view))
+         && (layer = gwy_data_view_get_base_layer(view))
+         && (key = gwy_pixmap_layer_get_data_key(layer))) {
+        gdouble min, max;
+        gwy_layer_basic_get_range(GWY_LAYER_BASIC(layer), &min, &max);
+        gwy_color_axis_set_range(GWY_COLOR_AXIS(data_window->coloraxis),
+                                 min, max);
+    }
 
     widget = GTK_WIDGET(data_window);
     GTK_WIDGET_CLASS(gwy_data_window_parent_class)->size_request(widget, &req);
