@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwydgetenums.h 13351 2012-03-29 09:23:12Z yeti-dn $
+ *  @(#) $Id: gwydgetenums.h 14725 2013-03-08 15:43:19Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -48,7 +48,8 @@ typedef enum { /*< lowercase_name=gwy_3d_projection >*/
 typedef enum { /*< lowercase_name=gwy_3d_visualization >*/
     GWY_3D_VISUALIZATION_GRADIENT = 0,
     GWY_3D_VISUALIZATION_LIGHTING,
-    GWY_3D_VISUALIZATION_OVERLAY
+    GWY_3D_VISUALIZATION_OVERLAY,
+    GWY_3D_VISUALIZATION_OVERLAY_NO_LIGHT
 } Gwy3DVisualization;
 
 typedef enum { /*< lowercase_name=gwy_3d_view_label >*/
