@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwygraphexport.c 12576 2011-07-11 14:51:57Z yeti-dn $
+ *  @(#) $Id: gwygraphexport.c 15067 2013-06-24 09:53:11Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -152,8 +152,8 @@ gwy_graph_export_postscript(GwyGraph *graph,
              /(gdouble)size*PANGO_SCALE;
     labelw = graph->area->lab->reqwidth*fontsize/(gdouble)size*PANGO_SCALE
              - 0.08*fontsize*fontsize;
-    labelx = width - areax - labelw - 5;
-    labely = height - areay - labelh - 5;
+    labelx = areax + areaw - labelw - 5;
+    labely = areay + areah - labelh - 5;
 
 
     /*write header*/
