@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwymarkerbox.c 12576 2011-07-11 14:51:57Z yeti-dn $
+ *  @(#) $Id: gwymarkerbox.c 14843 2013-04-10 10:26:27Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -291,6 +291,10 @@ gwy_marker_box_realize(GtkWidget *widget)
                                     &attributes, attributes_mask);
     gdk_window_set_user_data(widget->window, widget);
 
+    /* Force widget to listen to relevant events.  Works around oxygen
+     * misdetection of ‘empty’ areas.  See KDE bug #317292. */
+    gtk_widget_add_events(widget, attributes.event_mask);
+
     widget->style = gtk_style_attach(widget->style, widget->window);
     gtk_style_set_background(widget->style, widget->window, GTK_STATE_NORMAL);
 }
