@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwystock.c 12576 2011-07-11 14:51:57Z yeti-dn $
+ *  @(#) $Id: gwystock.c 13635 2012-07-31 08:25:06Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -38,7 +38,6 @@ static void           register_icon_set_list_cb(const gchar *id,
 static GtkIconSource* file_to_icon_source      (const gchar *path,
                                                 const gchar *filename,
                                                 gchar **id);
-static void           free_the_icon_factory    (void);
 
 static GtkIconFactory *the_icon_factory = NULL;
 
@@ -70,7 +69,6 @@ gwy_stock_register_stock_items(void)
     g_free(pixmap_paths[0]);
     g_free(pixmap_paths[1]);
     gtk_icon_factory_add_default(the_icon_factory);
-    g_atexit(free_the_icon_factory);
 }
 
 static void
@@ -259,12 +257,6 @@ file_to_icon_source(const gchar *path,
     return icon_source;
 }
 
-static void
-free_the_icon_factory(void)
-{
-    gwy_object_unref(the_icon_factory);
-}
-
 /************************** Documentation ****************************/
 
 /**
@@ -890,6 +882,15 @@ free_the_icon_factory(void)
  * <inlinegraphic fileref="gwy_polynom-24.png" format="PNG"/>
  **/
 
+/**
+ * GWY_STOCK_POLYNOM_LEVEL:
+ *
+ * The "Polynom-Level" stock icon.
+ * <inlinegraphic fileref="gwy_polynom_level-24.png" format="PNG"/>
+ *
+ * Since: 2.29
+ **/
+
 /**
  * GWY_STOCK_PROFILE:
  *
