@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: correct.c 13136 2011-12-06 13:56:51Z yeti-dn $
+ *  @(#) $Id: correct.c 14632 2013-02-12 17:03:20Z yeti-dn $
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -670,7 +670,7 @@ gwy_data_field_distort(GwyDataField *source,
                 v = gwy_interpolation_interpolate_2d(x, y, suplen, coeff,
                                                      interp);
             }
-            data[newj + xres*newi] = v;
+            data[newj + newxres*newi] = v;
         }
     }
 
