@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwygrainvalue.c 12576 2011-07-11 14:51:57Z yeti-dn $
+ *  @(#) $Id: gwygrainvalue.c 13724 2012-08-22 09:58:19Z yeti-dn $
  *  Copyright (C) 2007 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -184,6 +184,15 @@ grain_values[] = {
             2, 0, 0, 0,
         }
     },
+    {
+        N_("Area of convex hull"),
+        GWY_GRAIN_VALUE_CONVEX_HULL_AREA,
+        {
+            GWY_GRAIN_VALUE_GROUP_AREA,
+            "<i>A</i><sub>c</sub>", "A_c", NULL,
+            2, 0, 0, 0,
+        }
+    },
     {
         /* TRANSLATORS: Grain volume calculated using basis (foot) set to
          * zero. */
@@ -262,6 +271,69 @@ grain_values[] = {
             0, 0, GWY_GRAIN_VALUE_IS_ANGLE, 0,
         }
     },
+    {
+        N_("Maximum inscribed disc radius"),
+        GWY_GRAIN_VALUE_INSCRIBED_DISC_R,
+        {
+            GWY_GRAIN_VALUE_GROUP_BOUNDARY,
+            "<i>R</i><sub>i</sub>", "R_i", NULL,
+            1, 0, 0, 0,
+        }
+    },
+    {
+        N_("Maximum inscribed disc center x position"),
+        GWY_GRAIN_VALUE_INSCRIBED_DISC_X,
+        {
+            GWY_GRAIN_VALUE_GROUP_BOUNDARY,
+            "<i>x</i><sub>i</sub>", "x_i", NULL,
+            1, 0, 0, 0,
+        }
+    },
+    {
+        N_("Maximum inscribed disc center y position"),
+        GWY_GRAIN_VALUE_INSCRIBED_DISC_Y,
+        {
+            GWY_GRAIN_VALUE_GROUP_BOUNDARY,
+            "<i>y</i><sub>i</sub>", "y_i", NULL,
+            1, 0, 0, 0,
+        }
+    },
+    {
+        N_("Minimum circumcircle radius"),
+        GWY_GRAIN_VALUE_CIRCUMCIRCLE_R,
+        {
+            GWY_GRAIN_VALUE_GROUP_BOUNDARY,
+            "<i>R</i><sub>e</sub>", "R_e", NULL,
+            1, 0, 0, 0,
+        }
+    },
+    {
+        N_("Minimum circumcircle center x position"),
+        GWY_GRAIN_VALUE_CIRCUMCIRCLE_X,
+        {
+            GWY_GRAIN_VALUE_GROUP_BOUNDARY,
+            "<i>x</i><sub>e</sub>", "x_e", NULL,
+            1, 0, 0, 0,
+        }
+    },
+    {
+        N_("Minimum circumcircle center y position"),
+        GWY_GRAIN_VALUE_CIRCUMCIRCLE_Y,
+        {
+            GWY_GRAIN_VALUE_GROUP_BOUNDARY,
+            "<i>y</i><sub>e</sub>", "y_e", NULL,
+            1, 0, 0, 0,
+        }
+    },
+    {
+        N_("Mean radius"),
+        GWY_GRAIN_VALUE_MEAN_RADIUS,
+        {
+            GWY_GRAIN_VALUE_GROUP_BOUNDARY,
+            "<i>R</i><sub>m</sub>", "R_m", NULL,
+            1, 0, 0, 0,
+        }
+    },
     {
         N_("Inclination θ"),
         GWY_GRAIN_VALUE_SLOPE_THETA,
