@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyprocessenums.h 13351 2012-03-29 09:23:12Z yeti-dn $
+ *  @(#) $Id: gwyprocessenums.h 13724 2012-08-22 09:58:19Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -184,6 +184,14 @@ typedef enum {
     GWY_GRAIN_VALUE_CURVATURE2           = 31,
     GWY_GRAIN_VALUE_CURVATURE_ANGLE1     = 32,
     GWY_GRAIN_VALUE_CURVATURE_ANGLE2     = 33,
+    GWY_GRAIN_VALUE_INSCRIBED_DISC_R     = 34,
+    GWY_GRAIN_VALUE_INSCRIBED_DISC_X     = 35,
+    GWY_GRAIN_VALUE_INSCRIBED_DISC_Y     = 36,
+    GWY_GRAIN_VALUE_CONVEX_HULL_AREA     = 37,
+    GWY_GRAIN_VALUE_CIRCUMCIRCLE_R       = 38,
+    GWY_GRAIN_VALUE_CIRCUMCIRCLE_X       = 39,
+    GWY_GRAIN_VALUE_CIRCUMCIRCLE_Y       = 40,
+    GWY_GRAIN_VALUE_MEAN_RADIUS          = 41,
 } GwyGrainQuantity;
 
 typedef enum {
