@@ -1,5 +1,5 @@
 /*
- *  $Id: ezdfile.c 12576 2011-07-11 14:51:57Z yeti-dn $
+ *  $Id: ezdfile.c 15388 2013-10-01 20:19:21Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -32,6 +32,14 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-FILEMAGIC]
+ * # Nanosurf EZD.
+ * 0 string [DataSet]\x0d\x0a
+ * >&0 string Version=
+ * >>&0 regex [0-9]+ Nanosurf EZD SPM data version %s
+ **/
+
 /**
  * [FILE-MAGIC-USERGUIDE]
  * Nanosurf EZD, NID
@@ -89,6 +97,7 @@ typedef struct {
     guint byteorder;
     gboolean sign;
     const gchar *data;
+    GError *firsterror;
 } EZDSection;
 
 static gboolean      module_register       (void);
@@ -100,6 +109,7 @@ static GwyContainer* ezdfile_load          (const gchar *filename,
 static guint         find_data_start       (const guchar *buffer,
                                             gsize size);
 static void          ezdfile_free          (GPtrArray *ezdfile);
+static void          ezdsection_free       (EZDSection *section);
 static void          read_data_field       (GwyDataField *dfield,
                                             EZDSection *section);
 static gboolean      file_read_header      (GPtrArray *ezdfile,
@@ -122,7 +132,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Nanosurf EZD and NID data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.7.1",
+    "0.8",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -259,19 +269,26 @@ ezdfile_free(GPtrArray *ezdfile)
 
     for (i = 0; i < ezdfile->len; i++) {
         section = (EZDSection*)g_ptr_array_index(ezdfile, i);
-        g_hash_table_destroy(section->meta);
-        g_free(section->name);
-        g_free(section->xrange.unit);
-        g_free(section->yrange.unit);
-        g_free(section->zrange.unit);
-        g_free(section->xrange.name);
-        g_free(section->yrange.name);
-        g_free(section->zrange.name);
-        g_free(section);
+        ezdsection_free(section);
     }
     g_ptr_array_free(ezdfile, TRUE);
 }
 
+static void
+ezdsection_free(EZDSection *section)
+{
+    g_hash_table_destroy(section->meta);
+    g_clear_error(&section->firsterror);
+    g_free(section->name);
+    g_free(section->xrange.unit);
+    g_free(section->yrange.unit);
+    g_free(section->zrange.unit);
+    g_free(section->xrange.name);
+    g_free(section->yrange.name);
+    g_free(section->zrange.name);
+    g_free(section);
+}
+
 static gboolean
 file_read_header(GPtrArray *ezdfile,
                  gchar *buffer,
@@ -316,21 +333,34 @@ file_read_header(GPtrArray *ezdfile,
         p++;
 
         if (gwy_strequal(line, "SaveMode")) {
-            if (strcmp(p, "Binary"))
-                g_warning("SaveMode is not Binary, this is not supported");
+            if (!gwy_strequal(p, "Binary")) {
+                if (!section->firsterror)
+                    err_UNSUPPORTED(&section->firsterror, "SaveMode");
+            }
         }
-        else if (gwy_strequal(line, "SaveBits"))
+        else if (gwy_strequal(line, "SaveBits")) {
             section->bitdepth = atol(p);
+            if (section->bitdepth != 8
+                && section->bitdepth != 16
+                && section->bitdepth != 32) {
+                if (!section->firsterror)
+                    err_BPP(&section->firsterror, section->bitdepth);
+            }
+        }
         else if (gwy_strequal(line, "SaveSign")) {
             section->sign = gwy_strequal(p, "Signed");
-            if (!section->sign)
-                g_warning("SaveSign is not Signed, this is not supported");
+            if (!section->sign) {
+                if (!section->firsterror)
+                    err_UNSUPPORTED(&section->firsterror, "SaveSign");
+            }
         }
         else if (gwy_strequal(line, "SaveOrder")) {
             if (gwy_strequal(p, "Intel"))
                 section->byteorder = G_LITTLE_ENDIAN;
-            else
-                g_warning("SaveOrder is not Intel, this is not supported");
+            else {
+                if (!section->firsterror)
+                    err_UNSUPPORTED(&section->firsterror, "SaveOrder");
+            }
         }
         else if (gwy_strequal(line, "Frame")) {
             if (gwy_strequal(p, "Scan forward"))
@@ -370,6 +400,7 @@ file_read_header(GPtrArray *ezdfile,
         else
             g_hash_table_replace(section->meta, g_strdup(line), g_strdup(p));
     }
+
     return TRUE;
 }
 
@@ -379,7 +410,7 @@ find_data_offsets(const gchar *buffer,
                   GPtrArray *ezdfile,
                   GError **error)
 {
-    EZDSection *dataset, *section;
+    EZDSection *dataset, *section, *firstchannel = NULL;
     GString *grkey;
     guint required_size = 0;
     gint ngroups, nchannels, i, j, k;
@@ -433,6 +464,8 @@ find_data_offsets(const gchar *buffer,
                 g_warning("Cannot find section for %s", p);
                 continue;
             }
+            if (!firstchannel)
+                firstchannel = section;
 
             /* Compute data position */
             gwy_debug("Data %s at offset %u from data start",
@@ -444,6 +477,11 @@ find_data_offsets(const gchar *buffer,
                 gwy_debug("Skipping 1D data Gr%d-Ch%d. FIXME.", i, j);
                 continue;
             }
+            if (section->firsterror) {
+                gwy_debug("Skipping data Gr%d-Ch%d due to previous errors.",
+                          i, j);
+                continue;
+            }
             ndata++;
             section->data = buffer + required_size;
             required_size += section->xres * section->yres
@@ -461,8 +499,12 @@ find_data_offsets(const gchar *buffer,
     }
     g_string_free(grkey, TRUE);
 
-    if (!ndata)
-        err_NO_DATA(error);
+    if (!ndata) {
+        if (firstchannel)
+            g_propagate_error(error, firstchannel->firsterror);
+        else
+            err_NO_DATA(error);
+    }
 
     return ndata;
 }
@@ -577,7 +619,7 @@ read_data_field(GwyDataField *dfield,
     gwy_data_field_resample(dfield, xres, yres, GWY_INTERPOLATION_NONE);
     data = gwy_data_field_get_data(dfield);
 
-    q = 1 << section->bitdepth;
+    q = pow(2.0, section->bitdepth);
     z0 = q/2.0;
     if (section->bitdepth == 8) {
         const gchar *p = section->data;
@@ -597,6 +639,16 @@ read_data_field(GwyDataField *dfield,
             }
         }
     }
+    else if (section->bitdepth == 32) {
+        const gint32 *p = (const gint32*)section->data;
+
+        for (i = 0; i < yres; i++) {
+            for (j = 0; j < xres; j++) {
+                data[i*xres + j] = GINT32_FROM_LE(p[(yres-1 - i)*xres + j]);
+                data[i*xres + j] = (data[i*xres + j] + z0)/q;
+            }
+        }
+    }
     else
         g_warning("Damn! Bit depth %d is not implemented", section->bitdepth);
 }
