@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwytiff.h 13464 2012-05-16 21:26:08Z yeti-dn $
+ *  @(#) $Id: gwytiff.h 15324 2013-09-15 19:38:31Z yeti-dn $
  *  Copyright (C) 2007 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -60,6 +60,7 @@ typedef enum {
 
 /* Standard TIFF tags */
 typedef enum {
+    GWY_TIFFTAG_SUB_FILE_TYPE     = 254,
     GWY_TIFFTAG_IMAGE_WIDTH       = 256,
     GWY_TIFFTAG_IMAGE_LENGTH      = 257,
     GWY_TIFFTAG_BITS_PER_SAMPLE   = 258,
@@ -107,15 +108,21 @@ typedef enum {
     GWY_TIFF_PHOTOMETRIC_RGB          = 2,
 } GwyTIFFPhotometric;
 
+typedef enum {
+    GWY_TIFF_SUBFILE_FULL_IMAGE_DATA    = 1,
+    GWY_TIFF_SUBFILE_REDUCED_IMAGE_DATA = 2,
+    GWY_TIFF_SUBFILE_SINGLE_PAGE        = 3,
+} GwyTIFFSubFileType;
+
 typedef enum {
     GWY_TIFF_PLANAR_CONFIG_CONTIGNUOUS = 1,
     GWY_TIFF_PLANAR_CONFIG_SEPARATE = 2,
 } GwyTIFFPlanarConfig;
 
 typedef enum {
-    GWY_TIFF_RESOLUTION_UNIT_NONE = 0,
-    GWY_TIFF_RESOLUTION_UNIT_INCH = 1,
-    GWY_TIFF_RESOLUTION_UNIT_CENTIMETER = 2,
+    GWY_TIFF_RESOLUTION_UNIT_NONE = 1,
+    GWY_TIFF_RESOLUTION_UNIT_INCH = 2,
+    GWY_TIFF_RESOLUTION_UNIT_CENTIMETER = 3,
 } GwyTIFFResolutionUnit;
 
 typedef enum {
