@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: nanoscope.c 14614 2013-02-08 08:57:52Z yeti-dn $
+ *  @(#) $Id: nanoscope.c 15314 2013-09-13 17:52:56Z yeti-dn $
  *  Copyright (C) 2004-2012 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -29,14 +29,21 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-FILEMAGIC]
+ * # Nanoscope III
+ * # Two header variants.
+ * 0 string \\*File\ list\x0d\x0a Nanoscope III SPM binary data
+ * 0 string ?*File\ list\x0d\x0a Nanoscope III SPM text data
+ **/
+
 /**
  * [FILE-MAGIC-USERGUIDE]
  * Veeco Nanoscope III
  * .001 .002 etc.
- * Read SPS:Limited[1]
+ * Read SPS:Limited[1] Volume
  * [1] Spectra curves are imported as graphs, positional information is lost.
  **/
-
 #include "config.h"
 #include <errno.h>
 #include <string.h>
@@ -69,6 +76,7 @@ typedef enum {
     NANOSCOPE_FILE_TYPE_BIN,
     NANOSCOPE_FILE_TYPE_TXT,
     NANOSCOPE_FILE_TYPE_FORCE_BIN,
+    NANOSCOPE_FILE_TYPE_FORCE_VOLUME,
     NANOSCOPE_FILE_TYPE_BROKEN
 } NanoscopeFileType;
 
@@ -107,6 +115,8 @@ typedef struct {
     GHashTable *hash;
     GwyDataField *data_field;
     GwyGraphModel *graph_model;
+    GwyBrick *brick;
+    GwyBrick *second_brick;
 } NanoscopeData;
 
 static gboolean        module_register        (void);
@@ -135,6 +145,15 @@ static GwyGraphModel*  hash_to_curve          (GHashTable *hash,
                                                gchar *buffer,
                                                gint gxres,
                                                GError **error);
+static GwyBrick*       hash_to_brick          (GHashTable *hash,
+                                               GHashTable *forcelist,
+                                               GHashTable *scanlist,
+                                               GHashTable *scannerlist,
+                                               NanoscopeFileType file_type,
+                                               guint bufsize,
+                                               gchar *buffer,
+                                               GwyBrick **second_brick,
+                                               GError **error);
 static gboolean        read_text_data         (guint n,
                                                gdouble *data,
                                                gchar **buffer,
@@ -173,6 +192,7 @@ static GwyContainer*   nanoscope_get_metadata (GHashTable *hash,
                                                GList *list);
 static NanoscopeValue* parse_value            (const gchar *key,
                                                gchar *line);
+static GwyDataField*   preview_from_brick     (GwyBrick *brick);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -233,10 +253,12 @@ nanoscope_load(const gchar *filename,
     NanoscopeData *ndata;
     NanoscopeValue *val;
     GHashTable *hash, *scannerlist = NULL, *scanlist = NULL, *forcelist = NULL,
-               *contrlist = NULL;
+               *contrlist = NULL, *equipmentlist = NULL;
     GList *l, *list = NULL;
     gint i, xres = 0, yres = 0;
     gboolean ok;
+    GwyDataField *preview;
+    gchar *prevkey, *titlekey;
 
     if (!g_file_get_contents(filename, &buffer, &size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
@@ -285,7 +307,12 @@ nanoscope_load(const gchar *filename,
         ndata = g_new0(NanoscopeData, 1);
         ndata->hash = hash;
         list = g_list_append(list, ndata);
+
+        val = g_hash_table_lookup(hash, "Start context");
+        if (val && gwy_strequal(val->hard_value_str, "FVOL"))
+            file_type = NANOSCOPE_FILE_TYPE_FORCE_VOLUME;
     }
+
     if (err) {
         g_propagate_error(error, err);
         ok = FALSE;
@@ -304,7 +331,11 @@ nanoscope_load(const gchar *filename,
             scannerlist = hash;
             continue;
         }
-        if (gwy_strequal(self, "File list")) {
+        if (gwy_strequal(self, "Equipment list")) {
+            equipmentlist = hash;
+            continue;
+        }
+         if (gwy_strequal(self, "File list")) {
             continue;
         }
         if (gwy_strequal(self, "Controller list")) {
@@ -316,15 +347,16 @@ nanoscope_load(const gchar *filename,
             get_scan_list_res(hash, &xres, &yres);
             scanlist = hash;
         }
-        if (gwy_stramong(self, "Ciao force list", NULL)) {
+         if (gwy_stramong(self, "Ciao force list", NULL)) {
             get_scan_list_res(hash, &xres, &yres);
             forcelist = hash;
         }
-        if (!gwy_stramong(self, "Ciao image list", "AFM image list",
+        if (!gwy_stramong(self, "AFM image list", "Ciao image list",
                           "STM image list", "NCAFM image list",
                           "Ciao force image list", "Image list", NULL))
             continue;
 
+        gwy_debug("processing hash %s", self);
         if (file_type == NANOSCOPE_FILE_TYPE_FORCE_BIN) {
             ndata->graph_model = hash_to_curve(hash, forcelist, scanlist,
                                                scannerlist,
@@ -334,6 +366,17 @@ nanoscope_load(const gchar *filename,
                                                error);
             ok = ok && ndata->graph_model;
         }
+        else if (file_type == NANOSCOPE_FILE_TYPE_FORCE_VOLUME) {
+            if (!gwy_strequal(self, "Ciao force image list"))
+                continue;
+
+            ndata->brick = hash_to_brick(hash, forcelist, scanlist, equipmentlist,
+                                         file_type,
+                                         size, buffer,
+                                         &ndata->second_brick,
+                                         error);
+            ok = ok && ndata->brick;
+        }
         else {
             ndata->data_field = hash_to_data_field(hash, scannerlist, scanlist,
                                                    contrlist,
@@ -389,6 +432,31 @@ nanoscope_load(const gchar *filename,
                 gwy_container_set_object(container, quark, ndata->graph_model);
                 i++;
             }
+            else if (ndata->brick) {
+                GQuark quark = gwy_app_get_brick_key_for_id(i+1);
+
+                preview = preview_from_brick(ndata->brick);
+                prevkey = g_strconcat(g_quark_to_string(quark), "/preview", NULL);
+                titlekey = g_strconcat(g_quark_to_string(quark), "/title", NULL);
+                gwy_container_set_object(container, quark, ndata->brick);
+                gwy_container_set_object(container, g_quark_from_string(prevkey), preview);
+                gwy_container_set_string(container, g_quark_from_string(titlekey), g_strdup("Approach"));
+
+                i++;
+
+                if (ndata->second_brick) {
+                    quark = gwy_app_get_brick_key_for_id(i+1);
+                    preview = preview_from_brick(ndata->second_brick);
+                    prevkey = g_strconcat(g_quark_to_string(quark), "/preview", NULL);
+                    titlekey = g_strconcat(g_quark_to_string(quark), "/title", NULL);
+                    gwy_container_set_object(container, quark, ndata->second_brick);
+                    gwy_container_set_object(container, g_quark_from_string(prevkey), preview);
+                    gwy_container_set_string(container, g_quark_from_string(titlekey), g_strdup("Retract"));
+                    i++;
+
+
+                }
+            }
         }
         if (!i)
             gwy_object_unref(container);
@@ -427,6 +495,7 @@ get_scan_list_res(GHashTable *hash,
     gwy_debug("Global xres, yres = %d, %d", *xres, *yres);
 }
 
+
 static void
 add_metadata(gpointer hkey,
              gpointer hvalue,
@@ -661,6 +730,175 @@ hash_to_data_field(GHashTable *hash,
     return dfield;
 }
 
+static GwyBrick*
+hash_to_brick(GHashTable *hash, GHashTable *forcelist, GHashTable *scanlist, GHashTable *scannerlist,
+              G_GNUC_UNUSED NanoscopeFileType file_type,
+              guint bufsize,
+              gchar *buffer,
+              GwyBrick **second_brick,
+              GError **error)
+{
+    GwyBrick *brick;
+    NanoscopeValue *val;
+    gdouble *adata, *rdata;
+    guint xres, yres, zres, offset, length, bpp, i, j, l, st;
+    gint16 *d;
+    gdouble q;
+    gdouble *storage;
+    gdouble newl;
+    gdouble *abuf, *rbuf;
+    gint floorv;
+    gboolean continuous = FALSE;
+    gdouble rest;
+
+    gwy_debug("Loading brick");
+
+    if (!require_keys(hash, error, "Samps/line", "Data offset", "Data length",
+                      NULL))
+        return NULL;
+    if (!require_keys(forcelist, error, "force/line", NULL))
+        return NULL;
+    if (!require_keys(scanlist, error, "Scan size", NULL))
+        return NULL;
+
+    if (val = g_hash_table_lookup(scanlist, "Capture Mode")) {
+        if (strstr(val->hard_value_str, "Continuous")) {
+            continuous = TRUE;
+        }
+    }
+
+
+    /* scan size */
+    val = g_hash_table_lookup(hash, "Data offset");
+    offset = GWY_ROUND(val->hard_value);
+    gwy_debug("data offset %u", offset);
+
+    val = g_hash_table_lookup(hash, "Data length");
+    length = GWY_ROUND(val->hard_value);
+    gwy_debug("data length %u", length);
+
+    val = g_hash_table_lookup(hash, "Samps/line");
+    zres = GWY_ROUND(val->hard_value);
+    gwy_debug("samples in force line %u", zres);
+
+    val = g_hash_table_lookup(forcelist, "force/line");
+    xres = GWY_ROUND(val->hard_value);
+    gwy_debug("force curves per line %u", xres);
+
+    val = g_hash_table_lookup(hash, "Bytes/pixel");
+    bpp = val ? GWY_ROUND(val->hard_value) : 2;
+    if (bpp != 2) {
+        err_UNSUPPORTED(error, "Bytes/pixel");
+        return NULL;
+    }
+
+    /* FIXME: It is not clear how we should get yres.  Just divide the total
+     * data size with all the known sizes and hope for the best. */
+    yres = length/(xres*zres*bpp*2);
+
+    if (err_DIMENSION(error, xres)
+        || err_DIMENSION(error, yres)
+        || err_DIMENSION(error, zres))
+        return NULL;
+
+    if (err_SIZE_MISMATCH(error, offset + length, bufsize, FALSE))
+        return NULL;
+
+
+    brick = gwy_brick_new(xres, yres, zres, xres, yres, zres, 0);
+    adata = gwy_brick_get_data(brick);
+
+    /*up to now it seems that second brick is always present*/
+
+    *second_brick = gwy_brick_new(xres, yres, zres, xres, yres, zres, 0);
+    rdata = gwy_brick_get_data(*second_brick);
+
+
+    //gwy_debug("brick size expected to be %dx%dx%d (two bricks loaded), data length %d, expected %d", xres, yres, zres, length, xres*yres*zres*bpp*2);
+
+    q = 1.0/(1 << (8*bpp));
+    d = (gint16*)(buffer + offset);
+
+    storage = (gdouble*) malloc((xres*yres*zres*2 + 100) * sizeof(gdouble));
+    st = 0;
+
+    for (i = 0; i < yres; i++) {
+        for (j = 0; j < xres; j++) {
+            /* Approach curves */
+            for (l = 0; l < zres; l++) {
+                storage[st++] = q*GINT16_FROM_LE(*d);
+                d++;
+            }
+            /* Retract curves */
+            for (l = 0; l < zres; l++) {
+                storage[st + zres-l-1] = q*GINT16_FROM_LE(*d);
+                d++;
+            }
+            st += zres;
+        }
+    }
+
+
+    if (continuous) {
+        st = 47; //ad hoc fix, this should be detected automaticallz
+    } else st = 0;
+   
+    /*split data again with this strange shift*/ 
+    for (i = 0; i < yres; i++) {
+        for (j = 0; j < xres; j++) {
+            // Approach curves 
+            for (l = 0; l < zres; l++) {
+                adata[(l*yres + i)*xres + j] = storage[st++];
+                if (adata[(l*yres + i)*xres + j]<-0.45) adata[(l*yres + i)*xres + j] = storage[st-2]; //strange PF/QNM value?
+            }
+            // Retract curves 
+            for (l = 0; l < zres; l++) {
+                rdata[((zres-l-1)*yres + i)*xres + j] = storage[st++];
+            }
+        }
+    }
+
+    if (continuous) {
+        /*remove sine distortion from the data*/
+        abuf = (gdouble *)g_malloc(zres*sizeof(gdouble));
+        rbuf = (gdouble *)g_malloc(zres*sizeof(gdouble));
+
+        for (i = 0; i < yres; i++) {
+            for (j = 0; j < xres; j++) {
+                for (l = 0; l < zres; l++) {
+                    newl = zres*(asin(2.0*(gdouble)l/(gdouble)zres-1.0)+G_PI/2)/G_PI;
+                    floorv = floor(newl);
+                    rest = newl - floorv;
+                    abuf[l] = (1.0 - rest)*adata[(floorv*yres + i)*xres + j] + rest*adata[((floorv+1)*yres + i)*xres + j];
+                    rbuf[l] = (1.0 - rest)*rdata[(floorv*yres + i)*xres + j] + rest*rdata[((floorv+1)*yres + i)*xres + j];
+                }
+                for (l = 0; l < zres; l++) {
+                    adata[(l*yres + i)*xres + j] = abuf[l];
+                    rdata[(l*yres + i)*xres + j] = rbuf[l];
+                }
+            }
+        }
+
+        g_free(abuf);
+        g_free(rbuf);
+
+    }
+    g_free(storage);
+    return brick;
+}
+
+static GwyDataField*
+preview_from_brick(GwyBrick *brick)
+{
+    GwyDataField *dfield = gwy_data_field_new(10, 10, 10, 10, FALSE);
+
+    gwy_brick_sum_plane(brick, dfield, 0, 0, 0, gwy_brick_get_xres(brick),
+                        gwy_brick_get_yres(brick), -1, FALSE);
+
+    return dfield;
+
+}
+
 #define CHECK_AND_APPLY(op, hash, key)                     \
         if (!(val = g_hash_table_lookup((hash), (key)))) { \
             err_MISSING_FIELD(error, (key));               \
@@ -832,7 +1070,6 @@ hash_to_curve(GHashTable *hash,
     bpp = val ? GWY_ROUND(val->hard_value) : 2;
 
     /* scan size */
-
     offset = size = 0;
     if (file_type == NANOSCOPE_FILE_TYPE_FORCE_BIN) {
         val = g_hash_table_lookup(hash, "Data offset");
@@ -985,7 +1222,6 @@ hash_to_curve(GHashTable *hash,
  * get \@SENS in Ciao scan list -> SOFT
  * the factor is HARD*SOFT
  */
-
 static GwySIUnit*
 get_spec_ordinate_scale(GHashTable *hash,
                         GHashTable *scanlist,
@@ -1310,7 +1546,6 @@ read_hash(gchar **buffer,
         && value->hard_value_units
         && g_ascii_isdigit(value->hard_value_units[0])) {
         NanoscopeValue *val;
-
         val = g_new0(NanoscopeValue, 1);
         val->hard_value = g_ascii_strtod(value->hard_value_units, NULL);
         val->hard_value_str = value->hard_value_units;
@@ -1481,4 +1716,3 @@ parse_value(const gchar *key, gchar *line)
 }
 
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
-
