@@ -1,6 +1,6 @@
 /*
- *  @(#) $Id: arithmetic.c 13809 2012-09-19 18:19:21Z yeti-dn $
- *  Copyright (C) 2003-2010 David Necas (Yeti).
+ *  @(#) $Id: arithmetic.c 15383 2013-10-01 09:28:34Z yeti-dn $
+ *  Copyright (C) 2003-2013 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -62,7 +62,7 @@ enum {
 };
 
 enum {
-    ARITHMETIC_NARGS = NARGS * ARITHMETIC_NVARS
+    ARITHMETIC_NARGS = NARGS * ARITHMETIC_NVARS + 2  /* 2 for x and y */
 };
 
 enum {
@@ -126,12 +126,15 @@ static void          arithmetic_show_state        (ArithmeticControls *controls,
                                                    const gchar *message);
 static const gchar*  arithmetic_check_fields      (ArithmeticArgs *args);
 static void          arithmetic_preview           (ArithmeticControls *controls);
-static GwyDataField* arithmetic_do                (ArithmeticArgs *args);
+static GwyDataField* arithmetic_do                (ArithmeticArgs *args,
+                                                   gint *id);
 static void          arithmetic_need_data         (const ArithmeticArgs *args,
                                                    gboolean *need_data);
 static void          arithmetic_update_history    (ArithmeticArgs *args);
 static void          arithmetic_fix_mask_field    (ArithmeticArgs *args,
                                                    GwyDataField *mfield);
+static GwyDataField* make_x                       (GwyDataField *dfield);
+static GwyDataField* make_y                       (GwyDataField *dfield);
 static GwyDataField* make_x_der                   (GwyDataField *dfield);
 static GwyDataField* make_y_der                   (GwyDataField *dfield);
 
@@ -143,7 +146,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Simple arithmetic operations with data fields."),
     "Yeti <yeti@gwyddion.net>",
-    "3.0",
+    "3.2",
     "David Nečas (Yeti)",
     "2004",
 };
@@ -189,18 +192,20 @@ arithmetic(GwyContainer *data, GwyRunType run)
     gwy_expr_define_constant(args.expr, "π", G_PI, NULL);
 
     if (arithmetic_dialog(data, id, &args)) {
-        GwyDataField *result = arithmetic_do(&args);
+        GwyDataField *result = arithmetic_do(&args, &id);
 
         newid = gwy_app_data_browser_add_data_field(result, data, TRUE);
         g_object_unref(result);
         gwy_app_set_data_field_title(data, newid, _("Calculated"));
         gwy_app_sync_data_items(data, data, id, newid, FALSE,
-                                GWY_DATA_ITEM_GRADIENT, 0);
+                                GWY_DATA_ITEM_GRADIENT,
+                                GWY_DATA_ITEM_REAL_SQUARE,
+                                0);
     }
     arithmetic_save_args(settings, &args);
     g_ptr_array_free(args.ok_masks, TRUE);
     gwy_expr_free(args.expr);
-    for (i = 0; i < NARGS*ARITHMETIC_NVARS; i++)
+    for (i = 0; i < ARITHMETIC_NARGS; i++)
         g_free(args.name[i]);
 }
 
@@ -246,7 +251,9 @@ arithmetic_dialog(GwyContainer *data,
     dfield = gwy_data_field_new(PREVIEW_SIZE, PREVIEW_SIZE, 1.0, 1.0, TRUE);
     gwy_container_set_object_by_name(controls.mydata, "/0/data", dfield);
     gwy_app_sync_data_items(data, controls.mydata, id, 0, FALSE,
-                            GWY_DATA_ITEM_GRADIENT, 0);
+                            GWY_DATA_ITEM_GRADIENT,
+                            GWY_DATA_ITEM_REAL_SQUARE,
+                            0);
     controls.view = gwy_data_view_new(controls.mydata);
     gwy_data_view_set_data_prefix(GWY_DATA_VIEW(controls.view), "/0/data");
     layer = gwy_layer_basic_new();
@@ -341,6 +348,8 @@ arithmetic_dialog(GwyContainer *data,
 
         row++;
     }
+    args->name[NARGS*ARITHMETIC_NVARS + 0] = g_strdup("x");
+    args->name[NARGS*ARITHMETIC_NVARS + 1] = g_strdup("y");
 
     hbox2 = gtk_hbox_new(FALSE, 6);
     gtk_table_attach(GTK_TABLE(table), hbox2, 0, 2, row, row+1,
@@ -578,12 +587,13 @@ static void
 arithmetic_preview(ArithmeticControls *controls)
 {
     GwyDataField *result;
+    gint id = -1;
 
     /* We can also get here by activation of the entry so check again. */
     if (controls->args->err != ARITHMETIC_OK)
         return;
 
-    result = arithmetic_do(controls->args);
+    result = arithmetic_do(controls->args, &id);
     g_return_if_fail(result);
 
     gwy_container_set_object_by_name(controls->mydata, "/0/data", result);
@@ -592,7 +602,7 @@ arithmetic_preview(ArithmeticControls *controls)
 }
 
 static GwyDataField*
-arithmetic_do(ArithmeticArgs *args)
+arithmetic_do(ArithmeticArgs *args, gint *id)
 {
     static MakeFieldFunc derivers[ARITHMETIC_NVARS] = {
         NULL, NULL, make_x_der, make_y_der,
@@ -633,6 +643,7 @@ arithmetic_do(ArithmeticArgs *args)
             n = gwy_data_field_get_xres(dfield)*gwy_data_field_get_yres(dfield);
             result = gwy_data_field_new_alike(dfield, FALSE);
             r = gwy_data_field_get_data(result);
+            *id = args->objects[i].id;
         }
     }
 
@@ -650,6 +661,7 @@ arithmetic_do(ArithmeticArgs *args)
             n = gwy_data_field_get_xres(dfield)*gwy_data_field_get_yres(dfield);
             result = gwy_data_field_new_alike(dfield, FALSE);
             r = gwy_data_field_get_data(result);
+            *id = args->objects[i % NARGS].id;
         }
         quark = gwy_app_get_mask_key_for_id(args->objects[i % NARGS].id);
         mfield = NULL;
@@ -676,6 +688,20 @@ arithmetic_do(ArithmeticArgs *args)
         d[args->pos[i]] = gwy_data_field_get_data_const(dfield);
     }
 
+    i = NARGS*ARITHMETIC_NVARS + 0;
+    if (args->pos[i]) {
+        dfield = make_x(data_fields[0]);
+        data_fields[i] = dfield;
+        d[args->pos[i]] = gwy_data_field_get_data_const(dfield);
+    }
+
+    i = NARGS*ARITHMETIC_NVARS + 1;
+    if (args->pos[i]) {
+        dfield = make_y(data_fields[0]);
+        data_fields[i] = dfield;
+        d[args->pos[i]] = gwy_data_field_get_data_const(dfield);
+    }
+
     /* Execute */
     gwy_expr_vector_execute(args->expr, n, d, r);
 
@@ -718,6 +744,12 @@ arithmetic_need_data(const ArithmeticArgs *args,
         if (args->pos[i])
             need_data[i % NARGS] = TRUE;
     }
+
+    // When x and y are needed, always take them from field 1.  This also
+    // ensures the expression is considered to be a field expression.
+    if (args->pos[NARGS*ARITHMETIC_NVARS + 0]
+        || args->pos[NARGS*ARITHMETIC_NVARS + 1])
+        need_data[0] = TRUE;
 }
 
 static void
@@ -743,6 +775,55 @@ arithmetic_fix_mask_field(ArithmeticArgs *args,
     g_ptr_array_add(args->ok_masks, mfield);
 }
 
+static GwyDataField*
+make_x(GwyDataField *dfield)
+{
+    GwyDataField *result;
+    gdouble dx, xoff;
+    guint xres, yres, i, j;
+    gdouble *data;
+
+    result = gwy_data_field_new_alike(dfield, FALSE);
+    xres = gwy_data_field_get_xres(dfield);
+    yres = gwy_data_field_get_yres(dfield);
+    dx = gwy_data_field_get_xmeasure(dfield);
+    xoff = gwy_data_field_get_xoffset(dfield);
+    data = gwy_data_field_get_data(result);
+
+    for (j = 0; j < xres; j++)
+        data[j] = (j + 0.5)*dx + xoff;
+
+    for (i = 1; i < yres; i++)
+        memcpy(data + i*xres, data, xres*sizeof(gdouble));
+
+    return result;
+}
+
+static GwyDataField*
+make_y(GwyDataField *dfield)
+{
+    GwyDataField *result;
+    gdouble dy, yoff;
+    guint xres, yres, i, j;
+    gdouble *data;
+
+    result = gwy_data_field_new_alike(dfield, FALSE);
+    xres = gwy_data_field_get_xres(dfield);
+    yres = gwy_data_field_get_yres(dfield);
+    dy = gwy_data_field_get_ymeasure(dfield);
+    yoff = gwy_data_field_get_yoffset(dfield);
+    data = gwy_data_field_get_data(result);
+
+    for (i = 0; i < yres; i++) {
+        gdouble y = (i + 0.5)*dy + yoff;
+        gdouble *rrow = data + i*xres;
+        for (j = 0; j < xres; j++, rrow++)
+            *rrow = y;
+    }
+
+    return result;
+}
+
 static GwyDataField*
 make_x_der(GwyDataField *dfield)
 {
