@@ -1,6 +1,6 @@
 /*
- *  @(#) $Id: calibrate.c 13809 2012-09-19 18:19:21Z yeti-dn $
- *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
+ *  @(#) $Id: calibrate.c 15384 2013-10-01 09:37:43Z yeti-dn $
+ *  Copyright (C) 2003-2013 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -34,6 +34,14 @@
 
 #define CALIBRATE_RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
 
+/* for compatibility checks */
+#define EPSILON 1e-6
+
+typedef struct {
+    GwyContainer *data;
+    gint id;
+} GwyDataObjectId;
+
 typedef struct {
     gdouble xratio;
     gdouble yratio;
@@ -41,6 +49,7 @@ typedef struct {
     gint xyexponent;
     gint zexponent;
     gboolean square;
+    gboolean new_channel;
     gdouble xreal;
     gdouble yreal;
     gdouble zreal;
@@ -60,73 +69,97 @@ typedef struct {
     gchar *xyunitorig;
     gchar *zunit;
     gchar *zunitorig;
+    GwyDataObjectId sizeid;
+    GwyDataObjectId targetid;
 } CalibrateArgs;
 
 typedef struct {
     CalibrateArgs *args;
+    GtkWidget *dialog;
     GtkObject *xratio;
     GtkObject *yratio;
     GtkObject *zratio;
+    GtkWidget *xratio_spin;
+    GtkWidget *xratio_label;
+    GtkWidget *yratio_spin;
+    GtkWidget *yratio_label;
+    GtkWidget *match_size;
+    GtkWidget *size_chooser;
     GtkWidget *xyexponent;
     GtkWidget *zexponent;
     GtkWidget *xpower10;
     GtkWidget *ypower10;
     GtkWidget *zpower10;
     GtkWidget *square;
+    GtkWidget *new_channel;
     GtkObject *xreal;
     GtkObject *yreal;
     GtkObject *zreal;
+    GtkWidget *xreal_spin;
+    GtkWidget *xreal_label;
+    GtkWidget *yreal_spin;
+    GtkWidget *yreal_label;
     GtkObject *x0;
     GtkObject *y0;
     GtkObject *zshift;
     gboolean in_update;
     GtkWidget *xyunits;
     GtkWidget *zunits;
+    GtkWidget *ok;
 } CalibrateControls;
 
-static gboolean    module_register           (void);
-static void        calibrate                 (GwyContainer *data,
-                                              GwyRunType run);
-static gboolean    calibrate_dialog          (CalibrateArgs *args,
-                                              GwyDataField *dfield);
-static void        dialog_reset              (CalibrateControls *controls,
-                                              CalibrateArgs *args);
-static void        xratio_changed_cb         (GtkAdjustment *adj,
-                                              CalibrateControls *controls);
-static void        yratio_changed_cb         (GtkAdjustment *adj,
-                                              CalibrateControls *controls);
-static void        zratio_changed_cb         (GtkAdjustment *adj,
-                                              CalibrateControls *controls);
-static void        xreal_changed_cb          (GtkAdjustment *adj,
-                                              CalibrateControls *controls);
-static void        yreal_changed_cb          (GtkAdjustment *adj,
-                                              CalibrateControls *controls);
-static void        x0_changed_cb             (GtkAdjustment *adj,
-                                              CalibrateControls *controls);
-static void        y0_changed_cb             (GtkAdjustment *adj,
-                                              CalibrateControls *controls);
-static void        zshift_changed_cb         (GtkAdjustment *adj,
-                                              CalibrateControls *controls);
-static void        zreal_changed_cb          (GtkAdjustment *adj,
-                                              CalibrateControls *controls);
-static void        square_changed_cb         (GtkWidget *check,
-                                              CalibrateControls *controls);
-static void        xyexponent_changed_cb     (GtkWidget *combo,
-                                              CalibrateControls *controls);
-static void        zexponent_changed_cb      (GtkWidget *combo,
-                                              CalibrateControls *controls);
-static void        calibrate_dialog_update   (CalibrateControls *controls,
-                                              CalibrateArgs *args);
-static void        calibrate_sanitize_args   (CalibrateArgs *args);
-static void        calibrate_load_args       (GwyContainer *container,
-                                              CalibrateArgs *args);
-static void        calibrate_save_args       (GwyContainer *container,
-                                              CalibrateArgs *args);
-static void        units_change_cb           (GtkWidget *button,
-                                              CalibrateControls *controls);
-static void        set_combo_from_unit       (GtkWidget *combo,
-                                              const gchar *str,
-                                              gint basepower);
+static gboolean module_register        (void);
+static void     calibrate              (GwyContainer *data,
+                                        GwyRunType run);
+static gboolean calibrate_dialog       (CalibrateArgs *args,
+                                        GwyDataField *dfield);
+static void     dialog_reset           (CalibrateControls *controls,
+                                        CalibrateArgs *args);
+static void     xratio_changed         (GtkAdjustment *adj,
+                                        CalibrateControls *controls);
+static void     yratio_changed         (GtkAdjustment *adj,
+                                        CalibrateControls *controls);
+static void     zratio_changed         (GtkAdjustment *adj,
+                                        CalibrateControls *controls);
+static void     xreal_changed          (GtkAdjustment *adj,
+                                        CalibrateControls *controls);
+static void     yreal_changed          (GtkAdjustment *adj,
+                                        CalibrateControls *controls);
+static void     x0_changed             (GtkAdjustment *adj,
+                                        CalibrateControls *controls);
+static void     y0_changed             (GtkAdjustment *adj,
+                                        CalibrateControls *controls);
+static void     zshift_changed         (GtkAdjustment *adj,
+                                        CalibrateControls *controls);
+static void     zreal_changed          (GtkAdjustment *adj,
+                                        CalibrateControls *controls);
+static void     square_changed         (GtkWidget *check,
+                                        CalibrateControls *controls);
+static void     new_channel_changed    (GtkWidget *check,
+                                        CalibrateControls *controls);
+static void     xyexponent_changed     (GtkWidget *combo,
+                                        CalibrateControls *controls);
+static void     zexponent_changed      (GtkWidget *combo,
+                                        CalibrateControls *controls);
+static void     calibrate_dialog_update(CalibrateControls *controls,
+                                        CalibrateArgs *args);
+static void     calibrate_sanitize_args(CalibrateArgs *args);
+static void     calibrate_load_args    (GwyContainer *container,
+                                        CalibrateArgs *args);
+static void     calibrate_save_args    (GwyContainer *container,
+                                        CalibrateArgs *args);
+static void     units_change           (GtkWidget *button,
+                                        CalibrateControls *controls);
+static void     match_size_changed     (GtkToggleButton *toggle,
+                                        CalibrateControls *controls);
+static void     size_channel_changed   (GwyDataChooser *toggle,
+                                        CalibrateControls *controls);
+static gboolean mould_filter(GwyContainer *data,
+             gint id,
+             gpointer user_data);
+static void     set_combo_from_unit    (GtkWidget *combo,
+                                        const gchar *str,
+                                        gint basepower);
 
 static const CalibrateArgs calibrate_defaults = {
     1.0,
@@ -135,8 +168,11 @@ static const CalibrateArgs calibrate_defaults = {
     -6,
     -6,
     TRUE,
+    TRUE,
     0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0, 0, 0, 0,
-    "m", "m", "m", "m"
+    "m", "m", "m", "m",
+    { NULL, 0 },
+    { NULL, 0 },
 };
 
 static GwyModuleInfo module_info = {
@@ -144,7 +180,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Recalibrates scan lateral dimensions or value range."),
     "Petr Klapetek <klapetek@gwyddion.net>",
-    "2.8",
+    "2.10",
     "David Nečas (Yeti) & Petr Klapetek",
     "2003",
 };
@@ -170,6 +206,7 @@ static void
 calibrate(GwyContainer *data, GwyRunType run)
 {
     GwyDataField *dfields[3];
+    GQuark quarks[3];
     GQuark quark;
     gint oldid, newid;
     CalibrateArgs args;
@@ -180,11 +217,16 @@ calibrate(GwyContainer *data, GwyRunType run)
     gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, dfields + 0,
                                      GWY_APP_MASK_FIELD, dfields + 1,
                                      GWY_APP_SHOW_FIELD, dfields + 2,
+                                     GWY_APP_DATA_FIELD_KEY, quarks + 0,
+                                     GWY_APP_MASK_FIELD_KEY, quarks + 1,
+                                     GWY_APP_SHOW_FIELD_KEY, quarks + 2,
                                      GWY_APP_DATA_FIELD_ID, &oldid,
                                      0);
     g_return_if_fail(dfields[0]);
 
     calibrate_load_args(gwy_app_settings_get(), &args);
+    args.targetid.data = data;
+    args.targetid.id = oldid;
     args.xorig = gwy_data_field_get_xreal(dfields[0]);
     args.yorig = gwy_data_field_get_yreal(dfields[0]);
     args.zorig = gwy_data_field_get_max(dfields[0])
@@ -205,20 +247,41 @@ calibrate(GwyContainer *data, GwyRunType run)
     siunitxy = gwy_data_field_get_si_unit_xy(dfields[0]);
     args.xyunitorig = gwy_si_unit_get_string(siunitxy,
                                              GWY_SI_UNIT_FORMAT_VFMARKUP);
-    args.xyunit = args.xyunitorig;
+    args.xyunit = g_strdup(args.xyunitorig);
     siunitz = gwy_data_field_get_si_unit_z(dfields[0]);
     args.zunitorig = gwy_si_unit_get_string(siunitz,
                                             GWY_SI_UNIT_FORMAT_VFMARKUP);
-    args.zunit = args.zunitorig;
+    args.zunit = g_strdup(args.zunitorig);
 
     if (run == GWY_RUN_INTERACTIVE) {
         ok = calibrate_dialog(&args, dfields[0]);
         calibrate_save_args(gwy_app_settings_get(), &args);
-        if (!ok)
+        if (!ok) {
+            g_free(args.xyunit);
+            g_free(args.xyunitorig);
+            g_free(args.zunit);
+            g_free(args.zunitorig);
             return;
+        }
     }
 
-    dfields[0] = gwy_data_field_duplicate(dfields[0]);
+    if (!args.new_channel) {
+        guint n = 1;
+        if (dfields[1]) {
+            n++;
+            if (dfields[2])
+                n++;
+        }
+        else if (dfields[2]) {
+            quarks[1] = quarks[2];
+            quarks[2] = 0;
+            n++;
+        }
+        gwy_app_undo_qcheckpointv(data, n, quarks);
+    }
+
+    if (args.new_channel)
+        dfields[0] = gwy_data_field_duplicate(dfields[0]);
 
     gwy_data_field_set_xreal(dfields[0], args.xreal);
     gwy_data_field_set_yreal(dfields[0], args.yreal);
@@ -238,7 +301,9 @@ calibrate(GwyContainer *data, GwyRunType run)
     }
 
     if (dfields[1]) {
-        dfields[1] = gwy_data_field_duplicate(dfields[1]);
+        if (args.new_channel)
+            dfields[1] = gwy_data_field_duplicate(dfields[1]);
+
         gwy_data_field_set_xreal(dfields[1], args.xreal);
         gwy_data_field_set_yreal(dfields[1], args.yreal);
         gwy_data_field_set_xoffset(dfields[1], args.x0);
@@ -250,7 +315,9 @@ calibrate(GwyContainer *data, GwyRunType run)
     }
 
     if (dfields[2]) {
-        dfields[2] = gwy_data_field_duplicate(dfields[2]);
+        if (args.new_channel)
+            dfields[2] = gwy_data_field_duplicate(dfields[2]);
+
         gwy_data_field_set_xreal(dfields[2], args.xreal);
         gwy_data_field_set_yreal(dfields[2], args.yreal);
         gwy_data_field_set_xoffset(dfields[2], args.x0);
@@ -261,32 +328,47 @@ calibrate(GwyContainer *data, GwyRunType run)
         }
     }
 
-    newid = gwy_app_data_browser_add_data_field(dfields[0], data, TRUE);
-    g_object_unref(dfields[0]);
-    gwy_app_sync_data_items(data, data, oldid, newid, FALSE,
-                            GWY_DATA_ITEM_GRADIENT,
-                            GWY_DATA_ITEM_RANGE,
-                            GWY_DATA_ITEM_MASK_COLOR,
-                            0);
-    if (dfields[1]) {
-        quark = gwy_app_get_mask_key_for_id(newid);
-        gwy_container_set_object(data, quark, dfields[1]);
-        g_object_unref(dfields[1]);
+    if (args.new_channel) {
+        newid = gwy_app_data_browser_add_data_field(dfields[0], data, TRUE);
+        g_object_unref(dfields[0]);
+        gwy_app_sync_data_items(data, data, oldid, newid, FALSE,
+                                GWY_DATA_ITEM_GRADIENT,
+                                GWY_DATA_ITEM_RANGE,
+                                GWY_DATA_ITEM_MASK_COLOR,
+                                0);
+        if (dfields[1]) {
+            quark = gwy_app_get_mask_key_for_id(newid);
+            gwy_container_set_object(data, quark, dfields[1]);
+            g_object_unref(dfields[1]);
+        }
+        if (dfields[2]) {
+            quark = gwy_app_get_show_key_for_id(newid);
+            gwy_container_set_object(data, quark, dfields[2]);
+            g_object_unref(dfields[2]);
+        }
+
+        gwy_app_set_data_field_title(data, newid, _("Recalibrated Data"));
     }
-    if (dfields[2]) {
-        quark = gwy_app_get_show_key_for_id(newid);
-        gwy_container_set_object(data, quark, dfields[2]);
-        g_object_unref(dfields[2]);
+    else {
+        guint i;
+
+        for (i = 0; i < 3; i++) {
+            if (dfields[i])
+                gwy_data_field_data_changed(dfields[i]);
+        }
     }
 
-    gwy_app_set_data_field_title(data, newid, _("Recalibrated Data"));
+    g_free(args.xyunit);
+    g_free(args.xyunitorig);
+    g_free(args.zunit);
+    g_free(args.zunitorig);
 }
 
 static gboolean
 calibrate_dialog(CalibrateArgs *args,
                  GwyDataField *dfield)
 {
-    enum { RESPONSE_RESET = 1 };
+    enum { RESPONSE_RESET = 1};
     GtkWidget *dialog, *spin, *table, *label;
     GwySIUnit *unit;
     CalibrateControls controls;
@@ -295,14 +377,16 @@ calibrate_dialog(CalibrateArgs *args,
     dialog = gtk_dialog_new_with_buttons(_("Dimensions and Units"), NULL, 0,
                                          _("_Reset"), RESPONSE_RESET,
                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                         GTK_STOCK_OK, GTK_RESPONSE_OK,
                                          NULL);
+    controls.dialog = dialog;
+
+    controls.ok = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_OK, GTK_RESPONSE_OK);
     gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
 
     controls.args = args;
     controls.in_update = TRUE;
 
-    table = gtk_table_new(14, 4, FALSE);
+    table = gtk_table_new(17, 4, FALSE);
     gtk_table_set_row_spacings(GTK_TABLE(table), 2);
     gtk_table_set_col_spacings(GTK_TABLE(table), 6);
     gtk_container_set_border_width(GTK_CONTAINER(table), 4);
@@ -315,7 +399,34 @@ calibrate_dialog(CalibrateArgs *args,
                      0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
     row++;
 
+    controls.match_size
+        = gtk_check_button_new_with_mnemonic(_("_Match pixel size:"));
+    gtk_table_attach(GTK_TABLE(table), controls.match_size,
+                     0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+    g_signal_connect(controls.match_size, "toggled",
+                     G_CALLBACK(match_size_changed), &controls);
+
+    controls.size_chooser = gwy_data_chooser_new_channels();
+    gtk_widget_set_sensitive(controls.size_chooser, FALSE);
+    gwy_data_chooser_set_filter(GWY_DATA_CHOOSER(controls.size_chooser),
+                                mould_filter, &args->targetid, NULL);
+    gtk_table_attach_defaults(GTK_TABLE(table), controls.size_chooser,
+                              1, 3, row, row+1);
+    if (gwy_data_chooser_get_active(GWY_DATA_CHOOSER(controls.size_chooser),
+                                    NULL)) {
+        g_signal_connect(controls.size_chooser, "changed",
+                         G_CALLBACK(size_channel_changed), &controls);
+    }
+    else {
+        gtk_widget_set_sensitive(controls.match_size, FALSE);
+        gtk_widget_set_no_show_all(label, TRUE);
+        gtk_widget_set_no_show_all(controls.match_size, TRUE);
+        gtk_widget_set_no_show_all(controls.size_chooser, TRUE);
+    }
+    row++;
+
     label = gtk_label_new_with_mnemonic(_("_X range:"));
+    controls.xreal_label = label;
     gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
     gtk_table_attach(GTK_TABLE(table), label,
                      0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
@@ -323,13 +434,14 @@ calibrate_dialog(CalibrateArgs *args,
     controls.xreal = gtk_adjustment_new(args->xreal/pow10(args->xyexponent),
                                         0.01, 10000, 1, 10, 0);
     spin = gtk_spin_button_new(GTK_ADJUSTMENT(controls.xreal), 1, 2);
+    controls.xreal_spin = spin;
     gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
     gtk_table_attach(GTK_TABLE(table), spin,
                      1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
 
     unit = gwy_data_field_get_si_unit_xy(dfield);
     controls.xyexponent
-        = gwy_combo_box_metric_unit_new(G_CALLBACK(xyexponent_changed_cb),
+        = gwy_combo_box_metric_unit_new(G_CALLBACK(xyexponent_changed),
                                         &controls, -15, 6, unit,
                                         args->xyexponent);
     gtk_table_attach(GTK_TABLE(table), controls.xyexponent, 2, 3, row, row+2,
@@ -343,6 +455,7 @@ calibrate_dialog(CalibrateArgs *args,
     row++;
 
     label = gtk_label_new_with_mnemonic(_("_Y range:"));
+    controls.yreal_label = label;
     gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
     gtk_table_attach(GTK_TABLE(table), label,
                      0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
@@ -350,6 +463,7 @@ calibrate_dialog(CalibrateArgs *args,
     controls.yreal = gtk_adjustment_new(args->yreal/pow10(args->xyexponent),
                                         0.01, 10000, 1, 10, 0);
     spin = gtk_spin_button_new(GTK_ADJUSTMENT(controls.yreal), 1, 2);
+    controls.yreal_spin = spin;
     gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
     gtk_table_attach(GTK_TABLE(table), spin,
                      1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
@@ -402,7 +516,7 @@ calibrate_dialog(CalibrateArgs *args,
                      0, 1, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
 
     controls.zreal = gtk_adjustment_new(args->zreal/pow10(args->zexponent),
-                                        0.01, 10000, 1, 10, 0);
+                                        -10000, 10000, 1, 10, 0);
     spin = gtk_spin_button_new(GTK_ADJUSTMENT(controls.zreal), 1, 2);
     gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
     gtk_table_attach(GTK_TABLE(table), spin,
@@ -410,7 +524,7 @@ calibrate_dialog(CalibrateArgs *args,
 
     unit = gwy_data_field_get_si_unit_z(dfield);
     controls.zexponent
-        = gwy_combo_box_metric_unit_new(G_CALLBACK(zexponent_changed_cb),
+        = gwy_combo_box_metric_unit_new(G_CALLBACK(zexponent_changed),
                                         &controls, -15, 6, unit,
                                         args->zexponent);
     gtk_table_attach(GTK_TABLE(table), controls.zexponent, 2, 3, row, row+1,
@@ -449,6 +563,8 @@ calibrate_dialog(CalibrateArgs *args,
     spin = gwy_table_attach_spinbutton(table, row,
                                        _("_X calibration factor:"), "",
                                        controls.xratio);
+    controls.xratio_spin = spin;
+    controls.xratio_label = gwy_table_get_child_widget(table, row, 0);
     controls.xpower10 = gwy_table_get_child_widget(table, row, 2);
     gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 3);
     row++;
@@ -457,11 +573,13 @@ calibrate_dialog(CalibrateArgs *args,
     spin = gwy_table_attach_spinbutton(table, row,
                                        _("_Y calibration factor:"), "",
                                        controls.yratio);
+    controls.yratio_spin = spin;
+    controls.yratio_label = gwy_table_get_child_widget(table, row, 0);
     controls.ypower10 = gwy_table_get_child_widget(table, row, 2);
     gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 3);
     row++;
 
-    controls.zratio = gtk_adjustment_new(args->zratio, 0.001, 1000, 0.1, 1, 0);
+    controls.zratio = gtk_adjustment_new(args->zratio, -1000, 1000, 0.1, 1, 0);
     spin = gwy_table_attach_spinbutton(table, row,
                                        _("_Z calibration factor:"), "",
                                        controls.zratio);
@@ -469,30 +587,44 @@ calibrate_dialog(CalibrateArgs *args,
     gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 3);
     row++;
 
+    /***** Options *****/
+    label = gwy_label_new_header(_("Options"));
+    gtk_table_attach(GTK_TABLE(table), label,
+                     0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+    row++;
+
+    controls.new_channel
+        = gtk_check_button_new_with_mnemonic(_("Create new channel"));
+    gtk_table_attach(GTK_TABLE(table), controls.new_channel,
+                     0, 3, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+    row++;
+
     g_signal_connect(controls.xreal, "value-changed",
-                     G_CALLBACK(xreal_changed_cb), &controls);
+                     G_CALLBACK(xreal_changed), &controls);
     g_signal_connect(controls.yreal, "value-changed",
-                     G_CALLBACK(yreal_changed_cb), &controls);
+                     G_CALLBACK(yreal_changed), &controls);
     g_signal_connect(controls.xyunits, "clicked",
-                     G_CALLBACK(units_change_cb), &controls);
+                     G_CALLBACK(units_change), &controls);
     g_signal_connect(controls.zunits, "clicked",
-                     G_CALLBACK(units_change_cb), &controls);
+                     G_CALLBACK(units_change), &controls);
     g_signal_connect(controls.x0, "value-changed",
-                     G_CALLBACK(x0_changed_cb), &controls);
+                     G_CALLBACK(x0_changed), &controls);
     g_signal_connect(controls.y0, "value-changed",
-                     G_CALLBACK(y0_changed_cb), &controls);
+                     G_CALLBACK(y0_changed), &controls);
     g_signal_connect(controls.zshift, "value-changed",
-                     G_CALLBACK(zshift_changed_cb), &controls);
+                     G_CALLBACK(zshift_changed), &controls);
     g_signal_connect(controls.zreal, "value-changed",
-                     G_CALLBACK(zreal_changed_cb), &controls);
+                     G_CALLBACK(zreal_changed), &controls);
     g_signal_connect(controls.xratio, "value-changed",
-                     G_CALLBACK(xratio_changed_cb), &controls);
+                     G_CALLBACK(xratio_changed), &controls);
     g_signal_connect(controls.yratio, "value-changed",
-                     G_CALLBACK(yratio_changed_cb), &controls);
+                     G_CALLBACK(yratio_changed), &controls);
     g_signal_connect(controls.zratio, "value-changed",
-                     G_CALLBACK(zratio_changed_cb), &controls);
+                     G_CALLBACK(zratio_changed), &controls);
     g_signal_connect(controls.square, "toggled",
-                     G_CALLBACK(square_changed_cb), &controls);
+                     G_CALLBACK(square_changed), &controls);
+    g_signal_connect(controls.new_channel, "toggled",
+                     G_CALLBACK(new_channel_changed), &controls);
 
     controls.in_update = FALSE;
     /* sync all fields */
@@ -533,6 +665,10 @@ dialog_reset(CalibrateControls *controls,
 {
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(controls->square),
                                  calibrate_defaults.square);
+    /* Don't reset this one
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(controls->new_channel),
+                                 calibrate_defaults.new_channel);
+                                 */
 
     set_combo_from_unit(controls->xyexponent,
                         args->xyunitorig, args->xyorigexp);
@@ -567,8 +703,8 @@ dialog_reset(CalibrateControls *controls,
 }
 
 static void
-xratio_changed_cb(GtkAdjustment *adj,
-                  CalibrateControls *controls)
+xratio_changed(GtkAdjustment *adj,
+               CalibrateControls *controls)
 {
     CalibrateArgs *args = controls->args;
 
@@ -588,8 +724,8 @@ xratio_changed_cb(GtkAdjustment *adj,
 }
 
 static void
-yratio_changed_cb(GtkAdjustment *adj,
-                  CalibrateControls *controls)
+yratio_changed(GtkAdjustment *adj,
+               CalibrateControls *controls)
 {
     CalibrateArgs *args = controls->args;
 
@@ -609,8 +745,8 @@ yratio_changed_cb(GtkAdjustment *adj,
 }
 
 static void
-zratio_changed_cb(GtkAdjustment *adj,
-                  CalibrateControls *controls)
+zratio_changed(GtkAdjustment *adj,
+               CalibrateControls *controls)
 {
     CalibrateArgs *args = controls->args;
 
@@ -620,14 +756,16 @@ zratio_changed_cb(GtkAdjustment *adj,
     controls->in_update = TRUE;
     args->zratio = gtk_adjustment_get_value(adj)
                    * pow10(args->zexponent - args->zorigexp);
+
     args->zreal = args->zratio * args->zorig;
+
     calibrate_dialog_update(controls, args);
     controls->in_update = FALSE;
 }
 
 static void
-xreal_changed_cb(GtkAdjustment *adj,
-                 CalibrateControls *controls)
+xreal_changed(GtkAdjustment *adj,
+              CalibrateControls *controls)
 {
     CalibrateArgs *args = controls->args;
 
@@ -647,8 +785,8 @@ xreal_changed_cb(GtkAdjustment *adj,
 }
 
 static void
-yreal_changed_cb(GtkAdjustment *adj,
-                 CalibrateControls *controls)
+yreal_changed(GtkAdjustment *adj,
+              CalibrateControls *controls)
 {
     CalibrateArgs *args = controls->args;
 
@@ -668,8 +806,8 @@ yreal_changed_cb(GtkAdjustment *adj,
 }
 
 static void
-x0_changed_cb(GtkAdjustment *adj,
-              CalibrateControls *controls)
+x0_changed(GtkAdjustment *adj,
+           CalibrateControls *controls)
 {
     CalibrateArgs *args = controls->args;
 
@@ -684,8 +822,8 @@ x0_changed_cb(GtkAdjustment *adj,
 }
 
 static void
-y0_changed_cb(GtkAdjustment *adj,
-              CalibrateControls *controls)
+y0_changed(GtkAdjustment *adj,
+           CalibrateControls *controls)
 {
     CalibrateArgs *args = controls->args;
 
@@ -700,8 +838,8 @@ y0_changed_cb(GtkAdjustment *adj,
 }
 
 static void
-zshift_changed_cb(GtkAdjustment *adj,
-                  CalibrateControls *controls)
+zshift_changed(GtkAdjustment *adj,
+               CalibrateControls *controls)
 {
     CalibrateArgs *args = controls->args;
 
@@ -716,8 +854,8 @@ zshift_changed_cb(GtkAdjustment *adj,
 }
 
 static void
-zreal_changed_cb(GtkAdjustment *adj,
-                 CalibrateControls *controls)
+zreal_changed(GtkAdjustment *adj,
+              CalibrateControls *controls)
 {
     CalibrateArgs *args = controls->args;
 
@@ -733,8 +871,8 @@ zreal_changed_cb(GtkAdjustment *adj,
 }
 
 static void
-square_changed_cb(GtkWidget *check,
-                  CalibrateControls *controls)
+square_changed(GtkWidget *check,
+               CalibrateControls *controls)
 {
     CalibrateArgs *args = controls->args;
 
@@ -754,8 +892,16 @@ square_changed_cb(GtkWidget *check,
 }
 
 static void
-xyexponent_changed_cb(GtkWidget *combo,
-                      CalibrateControls *controls)
+new_channel_changed(GtkWidget *check,
+                    CalibrateControls *controls)
+{
+    CalibrateArgs *args = controls->args;
+    args->new_channel = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check));
+}
+
+static void
+xyexponent_changed(GtkWidget *combo,
+                   CalibrateControls *controls)
 {
     CalibrateArgs *args = controls->args;
 
@@ -779,8 +925,8 @@ xyexponent_changed_cb(GtkWidget *combo,
 }
 
 static void
-zexponent_changed_cb(GtkWidget *combo,
-                     CalibrateControls *controls)
+zexponent_changed(GtkWidget *combo,
+                  CalibrateControls *controls)
 {
     CalibrateArgs *args = controls->args;
 
@@ -798,8 +944,8 @@ zexponent_changed_cb(GtkWidget *combo,
 }
 
 static void
-units_change_cb(GtkWidget *button,
-                CalibrateControls *controls)
+units_change(GtkWidget *button,
+             CalibrateControls *controls)
 {
     GtkWidget *dialog, *hbox, *label, *entry;
     const gchar *id, *unit;
@@ -843,10 +989,12 @@ units_change_cb(GtkWidget *button,
     unit = gtk_entry_get_text(GTK_ENTRY(entry));
     if (gwy_strequal(id, "xy")) {
         set_combo_from_unit(controls->xyexponent, unit, 0);
+        g_free(controls->args->xyunit);
         controls->args->xyunit = g_strdup(unit);
     }
     else if (gwy_strequal(id, "z")) {
         set_combo_from_unit(controls->zexponent, unit, 0);
+        g_free(controls->args->zunit);
         controls->args->zunit = g_strdup(unit);
     }
 
@@ -856,6 +1004,83 @@ units_change_cb(GtkWidget *button,
     controls->in_update = FALSE;
 }
 
+static void
+match_size_changed(GtkToggleButton *toggle,
+                   CalibrateControls *controls)
+{
+    gboolean matching = gtk_toggle_button_get_active(toggle);
+
+    gtk_widget_set_sensitive(controls->size_chooser, matching);
+    gtk_widget_set_sensitive(controls->xreal_spin, !matching);
+    gtk_widget_set_sensitive(controls->xreal_label, !matching);
+    gtk_widget_set_sensitive(controls->yreal_spin, !matching);
+    gtk_widget_set_sensitive(controls->yreal_label, !matching);
+    gtk_widget_set_sensitive(controls->xratio_spin, !matching);
+    gtk_widget_set_sensitive(controls->xratio_label, !matching);
+    gtk_widget_set_sensitive(controls->yratio_spin, !matching);
+    gtk_widget_set_sensitive(controls->yratio_label, !matching);
+    gtk_widget_set_sensitive(controls->xyunits, !matching);
+    gtk_widget_set_sensitive(controls->xyexponent, !matching);
+    gtk_widget_set_sensitive(controls->square, !matching);
+    /* This is fixed to sensitive again in size_channel_changed() any channel
+     * is selected. */
+    gtk_dialog_set_response_sensitive(GTK_DIALOG(controls->dialog),
+                                      GTK_RESPONSE_OK, !matching);
+
+    if (matching)
+        size_channel_changed(GWY_DATA_CHOOSER(controls->size_chooser),
+                             controls);
+}
+
+static void
+size_channel_changed(GwyDataChooser *chooser,
+                     CalibrateControls *controls)
+{
+    CalibrateArgs *args = controls->args;
+    GwyDataField *mould;
+    GQuark quark;
+    gdouble dx, dy;
+    GwySIUnit *unit;
+
+    args->sizeid.data = gwy_data_chooser_get_active(chooser, &args->sizeid.id);
+    if (!args->sizeid.data) {
+        gtk_dialog_set_response_sensitive(GTK_DIALOG(controls->dialog),
+                                          GTK_RESPONSE_OK, FALSE);
+        return;
+    }
+
+    quark = gwy_app_get_data_key_for_id(args->sizeid.id);
+    mould = GWY_DATA_FIELD(gwy_container_get_object(args->sizeid.data, quark));
+    dx = gwy_data_field_get_xmeasure(mould);
+    dy = gwy_data_field_get_ymeasure(mould);
+    unit = gwy_data_field_get_si_unit_xy(mould);
+
+    controls->in_update = TRUE;
+    args->xreal = dx * args->xres;
+    args->yreal = dy * args->yres;
+    args->xratio = args->xreal/args->xorig;
+    args->yratio = args->yreal/args->yorig;
+    args->xyexponent = 3*floor(log10(args->xreal*args->yreal)/6);
+    g_free(args->xyunit);
+    args->xyunit = gwy_si_unit_get_string(unit, GWY_SI_UNIT_FORMAT_PLAIN);
+    args->square = (fabs(log(dx/dy)) <= EPSILON);
+
+    set_combo_from_unit(controls->xyexponent, args->xyunit, args->xyexponent);
+    gwy_enum_combo_box_set_active(GTK_COMBO_BOX(controls->xyexponent),
+                                  args->xyexponent);
+    calibrate_dialog_update(controls, args);
+    controls->in_update = FALSE;
+}
+
+static gboolean
+mould_filter(GwyContainer *data,
+             gint id,
+             gpointer user_data)
+{
+    GwyDataObjectId *object = (GwyDataObjectId*)user_data;
+    return data != object->data || id != object->id;
+}
+
 static void
 set_combo_from_unit(GtkWidget *combo,
                     const gchar *str,
@@ -904,6 +1129,8 @@ calibrate_dialog_update(CalibrateControls *controls,
 
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(controls->square),
                                  args->square);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(controls->new_channel),
+                                 args->new_channel);
 
     e = args->xyexponent - args->xyorigexp;
     if (!e)
@@ -919,13 +1146,19 @@ calibrate_dialog_update(CalibrateControls *controls,
     else
         g_snprintf(buffer, sizeof(buffer), "× 10<sup>%d</sup>", e);
     gtk_label_set_markup(GTK_LABEL(controls->zpower10), buffer);
+
+    if (args->zratio == 0 || args->zreal == 0)
+        gtk_widget_set_sensitive(controls->ok, FALSE);
+    else
+        gtk_widget_set_sensitive(controls->ok, TRUE);
 }
 
-static const gchar xratio_key[] = "/module/calibrate/xratio";
-static const gchar yratio_key[] = "/module/calibrate/yratio";
-static const gchar zratio_key[] = "/module/calibrate/zratio";
-static const gchar zshift_key[] = "/module/calibrate/zshift";
-static const gchar square_key[] = "/module/calibrate/square";
+static const gchar xratio_key[]      = "/module/calibrate/xratio";
+static const gchar yratio_key[]      = "/module/calibrate/yratio";
+static const gchar zratio_key[]      = "/module/calibrate/zratio";
+static const gchar zshift_key[]      = "/module/calibrate/zshift";
+static const gchar square_key[]      = "/module/calibrate/square";
+static const gchar new_channel_key[] = "/module/calibrate/new_channel";
 
 static void
 calibrate_sanitize_args(CalibrateArgs *args)
@@ -935,6 +1168,7 @@ calibrate_sanitize_args(CalibrateArgs *args)
     args->zratio = CLAMP(args->zratio, 1e-15, 1e15);
     args->zshift = CLAMP(args->zshift, -1e-9, 1e9);
     args->square = !!args->square;
+    args->new_channel = !!args->new_channel;
 }
 
 static void
@@ -948,6 +1182,8 @@ calibrate_load_args(GwyContainer *container,
     gwy_container_gis_double_by_name(container, zratio_key, &args->zratio);
     gwy_container_gis_double_by_name(container, zshift_key, &args->zshift);
     gwy_container_gis_boolean_by_name(container, square_key, &args->square);
+    gwy_container_gis_boolean_by_name(container, new_channel_key,
+                                      &args->new_channel);
     calibrate_sanitize_args(args);
 }
 
@@ -960,6 +1196,8 @@ calibrate_save_args(GwyContainer *container,
     gwy_container_set_double_by_name(container, zratio_key, args->zratio);
     gwy_container_set_double_by_name(container, zshift_key, args->zshift);
     gwy_container_set_boolean_by_name(container, square_key, args->square);
+    gwy_container_set_boolean_by_name(container, new_channel_key,
+                                      args->new_channel);
 }
 
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
