@@ -1,6 +1,6 @@
 /*
- *  @(#) $Id: grain_dist.c 12576 2011-07-11 14:51:57Z yeti-dn $
- *  Copyright (C) 2003-2008 David Necas (Yeti), Petr Klapetek.
+ *  @(#) $Id: grain_dist.c 13809 2012-09-19 18:19:21Z yeti-dn $
+ *  Copyright (C) 2003-2012 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -35,6 +35,7 @@
 
 #define DIST_RUN_MODES GWY_RUN_INTERACTIVE
 #define STAT_RUN_MODES (GWY_RUN_IMMEDIATE | GWY_RUN_INTERACTIVE)
+#define INSCRIBE_RUN_MODES GWY_RUN_IMMEDIATE
 
 enum {
     MIN_RESOLUTION = 4,
@@ -98,35 +99,39 @@ typedef struct {
     gboolean add_comment;
 } GrainDistExportData;
 
-static gboolean module_register                 (void);
-static void grain_dist                          (GwyContainer *data,
-                                                 GwyRunType run);
-static void grain_stat                          (GwyContainer *data,
-                                                 GwyRunType run);
-static GtkWidget* grain_stats_add_aux_button    (GtkWidget *hbox,
-                                                 const gchar *stock_id,
-                                                 const gchar *tooltip);
-static void grain_dist_dialog                   (GrainDistArgs *args,
-                                                 GwyContainer *data,
-                                                 GwyDataField *dfield,
-                                                 GwyDataField *mfield);
-static void mode_changed_cb                     (GtkToggleButton *button,
-                                                 GrainDistControls *controls);
-static void selected_changed_cb                 (GrainDistControls *controls);
-static void grain_dist_dialog_update_values     (GrainDistControls *controls,
-                                                 GrainDistArgs *args);
-static void grain_dist_dialog_update_sensitivity(GrainDistControls *controls,
-                                                 GrainDistArgs *args);
-static void grain_dist_run                      (GrainDistArgs *args,
-                                                 GwyContainer *data,
-                                                 GwyDataField *dfield,
-                                                 GwyDataField *mfield);
-static gchar* grain_dist_export_create          (gpointer user_data,
-                                                 gssize *data_len);
-static void grain_dist_load_args                (GwyContainer *container,
-                                                 GrainDistArgs *args);
-static void grain_dist_save_args                (GwyContainer *container,
-                                                 GrainDistArgs *args);
+static gboolean   module_register                     (void);
+static void       grain_dist                          (GwyContainer *data,
+                                                       GwyRunType run);
+static void       grain_stat                          (GwyContainer *data,
+                                                       GwyRunType run);
+static void       grain_inscribe_discs                (GwyContainer *data,
+                                                       GwyRunType run);
+static void       grain_exscribe_circles              (GwyContainer *data,
+                                                       GwyRunType run);
+static GtkWidget* grain_stats_add_aux_button          (GtkWidget *hbox,
+                                                       const gchar *stock_id,
+                                                       const gchar *tooltip);
+static void       grain_dist_dialog                   (GrainDistArgs *args,
+                                                       GwyContainer *data,
+                                                       GwyDataField *dfield,
+                                                       GwyDataField *mfield);
+static void       mode_changed_cb                     (GtkToggleButton *button,
+                                                       GrainDistControls *controls);
+static void       selected_changed_cb                 (GrainDistControls *controls);
+static void       grain_dist_dialog_update_values     (GrainDistControls *controls,
+                                                       GrainDistArgs *args);
+static void       grain_dist_dialog_update_sensitivity(GrainDistControls *controls,
+                                                       GrainDistArgs *args);
+static void       grain_dist_run                      (GrainDistArgs *args,
+                                                       GwyContainer *data,
+                                                       GwyDataField *dfield,
+                                                       GwyDataField *mfield);
+static gchar*     grain_dist_export_create            (gpointer user_data,
+                                                       gssize *data_len);
+static void       grain_dist_load_args                (GwyContainer *container,
+                                                       GrainDistArgs *args);
+static void       grain_dist_save_args                (GwyContainer *container,
+                                                       GrainDistArgs *args);
 
 static const GrainDistArgs grain_dist_defaults = {
     MODE_GRAPH,
@@ -144,7 +149,7 @@ static GwyModuleInfo module_info = {
     N_("Evaluates distribution of grains (continuous parts of mask)."),
     "Petr Klapetek <petr@klapetek.cz>, Sven Neumann <neumann@jpk.com>, "
         "Yeti <yeti@gwyddion.net>",
-    "3.8",
+    "3.10",
     "David Nečas (Yeti) & Petr Klapetek & Sven Neumann",
     "2003",
 };
@@ -176,6 +181,22 @@ module_register(void)
                               STAT_RUN_MODES,
                               GWY_MENU_FLAG_DATA | GWY_MENU_FLAG_DATA_MASK,
                               N_("Simple grain statistics"));
+    gwy_process_func_register("grain_inscribe_discs",
+                              (GwyProcessFunc)&grain_inscribe_discs,
+                              N_("/_Grains/Select _Inscribed Discs"),
+                              NULL,
+                              INSCRIBE_RUN_MODES,
+                              GWY_MENU_FLAG_DATA | GWY_MENU_FLAG_DATA_MASK,
+                              N_("Create a selection visualising discs "
+                                 "incribed into grains"));
+    gwy_process_func_register("grain_exscribe_circles",
+                              (GwyProcessFunc)&grain_exscribe_circles,
+                              N_("/_Grains/Select _Excscribed Circles"),
+                              NULL,
+                              INSCRIBE_RUN_MODES,
+                              GWY_MENU_FLAG_DATA | GWY_MENU_FLAG_DATA_MASK,
+                              N_("Create a selection visualising grain "
+                                 "circumcircles"));
 
     return TRUE;
 }
@@ -238,7 +259,6 @@ grain_dist_dialog(GrainDistArgs *args,
                                                     GTK_RESPONSE_CANCEL,
                                                     NULL));
     controls.ok = gtk_dialog_add_button(dialog, GTK_STOCK_OK, GTK_RESPONSE_OK);
-    gtk_dialog_set_has_separator(dialog, FALSE);
     gtk_dialog_set_default_response(dialog, GTK_RESPONSE_OK);
     gtk_window_set_default_size(GTK_WINDOW(dialog), -1, 520);
 
@@ -675,7 +695,7 @@ grain_stat(GwyContainer *data, GwyRunType run)
     GwySIUnit *siunit, *siunit2;
     GwySIValueFormat *vf;
     gint xres, yres, ngrains;
-    gdouble total_area, area, size, vol_0, vol_min, vol_laplace, v;
+    gdouble total_area, area, size, vol_0, vol_min, vol_laplace, bound_len, v;
     gdouble *values = NULL;
     gint *grains;
     GString *str;
@@ -728,15 +748,16 @@ grain_stat(GwyContainer *data, GwyRunType run)
                                      GWY_GRAIN_VALUE_VOLUME_MIN);
     vol_laplace = grains_get_total_value(dfield, ngrains, grains, &values,
                                          GWY_GRAIN_VALUE_VOLUME_LAPLACE);
+    bound_len = grains_get_total_value(dfield, ngrains, grains, &values,
+                                       GWY_GRAIN_VALUE_FLAT_BOUNDARY_LENGTH);
     g_free(values);
     g_free(grains);
 
     dialog = gtk_dialog_new_with_buttons(_("Grain Statistics"), NULL, 0,
                                          GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                          NULL);
-    gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
 
-    table = gtk_table_new(9, 2, FALSE);
+    table = gtk_table_new(10, 2, FALSE);
     gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);
     gtk_container_set_border_width(GTK_CONTAINER(table), 4);
     row = 0;
@@ -792,6 +813,12 @@ grain_stat(GwyContainer *data, GwyRunType run)
     add_report_row(GTK_TABLE(table), &row, _("Total grain volume (laplacian):"),
                    str->str, report);
 
+    v = bound_len;
+    gwy_si_unit_get_format(siunit, GWY_SI_UNIT_FORMAT_VFMARKUP, v, vf);
+    g_string_printf(str, "%.*f %s", vf->precision, v/vf->magnitude, vf->units);
+    add_report_row(GTK_TABLE(table), &row, _("Total projected boundary length:"),
+                   str->str, report);
+
     gwy_si_unit_value_format_free(vf);
     g_object_unref(siunit2);
 
@@ -901,4 +928,125 @@ grain_dist_save_args(GwyContainer *container,
     gwy_container_set_enum_by_name(container, mode_key, args->mode);
 }
 
+static GwySelection*
+create_selection(const gchar *typename,
+                 guint *ngrains)
+{
+    GParamSpecInt *pspec;
+    GObjectClass *klass;
+    GType type;
+
+    type = g_type_from_name(typename);
+    g_return_val_if_fail(type, NULL);
+
+    klass = g_type_class_ref(type);
+    pspec = (GParamSpecInt*)g_object_class_find_property(klass, "max-objects");
+    g_return_val_if_fail(G_IS_PARAM_SPEC_UINT(pspec), NULL);
+
+    if ((gint)*ngrains > pspec->maximum) {
+        g_warning("Too many grains for %s, only first %d will be shown.",
+                  typename, pspec->maximum);
+        *ngrains = pspec->maximum;
+    }
+    return g_object_new(type, "max-objects", *ngrains, NULL);
+}
+
+/* FIXME: It would be nice to have something like that also for minimum and
+ * maximum bounding dimensions. */
+static void
+grain_inscribe_discs(GwyContainer *data, GwyRunType run)
+{
+    static const GwyGrainQuantity quantities[] = {
+        GWY_GRAIN_VALUE_INSCRIBED_DISC_R,
+        GWY_GRAIN_VALUE_INSCRIBED_DISC_X,
+        GWY_GRAIN_VALUE_INSCRIBED_DISC_Y,
+    };
+
+    GwyDataField *dfield, *mfield;
+    GwySelection *selection;
+    guint ngrains, i;
+    gint *grains;
+    gdouble *inscd;
+    gdouble *values[3];
+    gchar *key;
+    gint id;
+
+    g_return_if_fail(run & INSCRIBE_RUN_MODES);
+    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield,
+                                     GWY_APP_MASK_FIELD, &mfield,
+                                     GWY_APP_DATA_FIELD_ID, &id,
+                                     0);
+
+    grains = g_new0(gint, mfield->xres * mfield->yres);
+    ngrains = gwy_data_field_number_grains(mfield, grains);
+    inscd = g_new(gdouble, 3*(ngrains + 1));
+    for (i = 0; i < 3; i++)
+        values[i] = inscd + i*(ngrains + 1);
+
+    gwy_data_field_grains_get_quantities(dfield, values, quantities, 3,
+                                         ngrains, grains);
+
+    selection = create_selection("GwySelectionEllipse", &ngrains);
+    for (i = 1; i <= ngrains; i++) {
+        gdouble r = values[0][i], x = values[1][i], y = values[2][i];
+        gdouble xy[4] = { x - r, y - r, x + r, y + r };
+        gwy_selection_set_object(selection, i-1, xy);
+    }
+
+    key = g_strdup_printf("/%d/select/ellipse", id);
+    gwy_container_set_object_by_name(data, key, selection);
+    g_object_unref(selection);
+
+    g_free(grains);
+    g_free(inscd);
+}
+
+static void
+grain_exscribe_circles(GwyContainer *data, GwyRunType run)
+{
+    static const GwyGrainQuantity quantities[] = {
+        GWY_GRAIN_VALUE_CIRCUMCIRCLE_R,
+        GWY_GRAIN_VALUE_CIRCUMCIRCLE_X,
+        GWY_GRAIN_VALUE_CIRCUMCIRCLE_Y,
+    };
+
+    GwyDataField *dfield, *mfield;
+    GwySelection *selection;
+    guint ngrains, i;
+    gint *grains;
+    gdouble *circc;
+    gdouble *values[3];
+    gchar *key;
+    gint id;
+
+    g_return_if_fail(run & INSCRIBE_RUN_MODES);
+    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield,
+                                     GWY_APP_MASK_FIELD, &mfield,
+                                     GWY_APP_DATA_FIELD_ID, &id,
+                                     0);
+
+    grains = g_new0(gint, mfield->xres * mfield->yres);
+    ngrains = gwy_data_field_number_grains(mfield, grains);
+    circc = g_new(gdouble, 3*(ngrains + 1));
+    for (i = 0; i < 3; i++)
+        values[i] = circc + i*(ngrains + 1);
+
+    gwy_data_field_grains_get_quantities(dfield, values, quantities, 3,
+                                         ngrains, grains);
+
+    selection = create_selection("GwySelectionEllipse", &ngrains);
+    for (i = 1; i <= ngrains; i++) {
+        gdouble r = values[0][i], x = values[1][i], y = values[2][i];
+        gdouble xy[4] = { x - r, y - r, x + r, y + r };
+        gwy_selection_set_object(selection, i-1, xy);
+    }
+
+    key = g_strdup_printf("/%d/select/ellipse", id);
+    gwy_container_set_object_by_name(data, key, selection);
+    g_object_unref(selection);
+
+    g_free(grains);
+    g_free(circc);
+}
+
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
