@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: mark_with.c 13809 2012-09-19 18:19:21Z yeti-dn $
+ *  @(#) $Id: mark_with.c 15174 2013-08-14 11:10:27Z yeti-dn $
  *  Copyright (C) 2009 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -801,7 +801,7 @@ static const gchar mark_with_key[] = "/module/mark_with/mark_with";
 static const gchar operation_key[] = "/module/mark_with/operation";
 static const gchar min_key[]       = "/module/mark_with/min";
 static const gchar max_key[]       = "/module/mark_with/max";
-static const gchar update_key[]    = "/module/fft_filter_1d/update";
+static const gchar update_key[]    = "/module/mask_with/update";
 
 static void
 mark_sanitize_args(MarkArgs *args)
