@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: noise_synth.c 13809 2012-09-19 18:19:21Z yeti-dn $
+ *  @(#) $Id: noise_synth.c 14942 2013-04-21 12:46:23Z yeti-dn $
  *  Copyright (C) 2010 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -209,7 +209,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Generates uncorrelated random noise."),
     "Yeti <yeti@gwyddion.net>",
-    "1.0",
+    "1.1",
     "David Nečas (Yeti)",
     "2010",
 };
@@ -304,9 +304,10 @@ run_noninteractive(NoiseSynthArgs *args,
     else {
         if (data) {
             newid = gwy_app_data_browser_add_data_field(dfield, data, TRUE);
-            gwy_app_sync_data_items(data, data, oldid, newid, FALSE,
-                                    GWY_DATA_ITEM_GRADIENT,
-                                    0);
+            if (oldid != -1)
+                gwy_app_sync_data_items(data, data, oldid, newid, FALSE,
+                                        GWY_DATA_ITEM_GRADIENT,
+                                        0);
         }
         else {
             newid = 0;
@@ -365,11 +366,10 @@ noise_synth_dialog(NoiseSynthArgs *args,
                                 TRUE);
     gwy_container_set_object_by_name(controls.mydata, "/0/data", dfield);
     controls.noise = gwy_data_field_new_alike(dfield, FALSE);
-    if (data)
+    if (dfield_template) {
         gwy_app_sync_data_items(data, controls.mydata, id, 0, FALSE,
                                 GWY_DATA_ITEM_PALETTE,
                                 0);
-    if (dfield_template) {
         controls.surface = gwy_synth_surface_for_preview(dfield_template,
                                                          PREVIEW_SIZE);
         controls.zscale = gwy_data_field_get_rms(dfield_template);
